@@ -17,9 +17,8 @@ with NML; if not, write to the Free Software Foundation, Inc.,
 Action 11 support classes (sounds).
 """
 import os
-from operator import itemgetter
 from nml import generic
-from nml.actions import base_action
+from nml.actions import base_action, action0
 
 class Action11(base_action.BaseAction):
     def __init__(self, num_sounds):
@@ -36,18 +35,17 @@ class LoadBinaryFile(base_action.BaseAction):
     '''
     <sprite-number> * <length> FF <name-len> <name> 00 <data>
     '''
-    def __init__(self, fname):
+    def __init__(self, fname, pos):
         self.fname = fname
+        self.pos = pos
         self.last = False
 
-    def prepare_output(self):
+    def prepare_output(self, sprite_num):
         if not os.access(self.fname, os.R_OK):
-            raise generic.ScriptError("File does not exist.", self.fname)
+            raise generic.ScriptError("Sound file '{}' does not exist.".format(self.fname), self.pos)
         size = os.path.getsize(self.fname)
         if size == 0:
-            raise generic.ScriptError("Expected a sound file with non-zero length.", self.fname)
-        if size > 0x10000:
-            raise generic.ScriptError("Sound file too big (max 64KB).", self.fname)
+            raise generic.ScriptError("Expected sound file '{}' to have a non-zero length.".format(self.fname), self.pos)
 
     def write(self, file):
         file.print_named_filedata(self.fname)
@@ -64,10 +62,14 @@ class ImportSound(base_action.BaseAction):
 
     @ivar number: Sound number to load.
     @type number: C{int}
+
+    @ivar pos: Position information
+    @type pos: L{Position}
     """
-    def __init__(self, grfid, number):
+    def __init__(self, grfid, number, pos):
         self.grfid = grfid
         self.number = number
+        self.pos = pos
         self.last = False
 
     def write(self, file):
@@ -80,11 +82,21 @@ class ImportSound(base_action.BaseAction):
         if self.last: file.newline()
 
 registered_sounds = {}
+SOUND_OFFSET = 73 # No of original sounds
+NUM_ANIMATION_SOUNDS = 0x80 - SOUND_OFFSET # Number of custom sound ids, which can be returned by animation callbacks.
 
-def add_sound(args):
+def print_stats():
+    """
+    Print statistics about used ids.
+    """
+    if len(registered_sounds) > 0:
+        # Currently NML does not optimise the order of sound effects. So we assume NUM_ANIMATION_SOUNDS as the maximum.
+        generic.print_info("Sound effects: {}/{}".format(len(registered_sounds), NUM_ANIMATION_SOUNDS))
+
+def add_sound(args, pos):
     if args not in registered_sounds:
-        registered_sounds[args] = len(registered_sounds)
-    return registered_sounds[args] + 73
+        registered_sounds[args] = (len(registered_sounds), pos)
+    return registered_sounds[args][0] + SOUND_OFFSET
 
 def get_sound_actions():
     """
@@ -95,11 +107,20 @@ def get_sound_actions():
 
     action_list = []
     action_list.append(Action11(len(registered_sounds)))
+    volume_list = []
 
-    for sound in map(itemgetter(0), sorted(registered_sounds.iteritems(), key=itemgetter(1))):
-        if isinstance(sound, tuple):
-            action_list.append(ImportSound(*sound))
+    sound_data = [(sound_id, args, pos) for args, (sound_id, pos) in list(registered_sounds.items())]
+    # Sort on first item, i.e. sound ID
+    for i, args, pos in sorted(sound_data):
+        if len(args) == 3:
+            action_list.append(ImportSound(args[0], args[1], pos))
         else:
-            action_list.append(LoadBinaryFile(sound))
+            action_list.append(LoadBinaryFile(args[0], pos))
+        if args[-1] != 100:
+            volume_list.append( (i + SOUND_OFFSET, int(args[-1] * 128 / 100)) )
+
+    if volume_list:
+        action_list.extend(action0.get_volume_actions(volume_list))
 
     return action_list
+
