@@ -18,18 +18,47 @@ from nml.actions import base_action
 from nml.ast import base_statement, general
 import nml
 
-free_action2_ids = list(range(1, 255))
+total_action2_ids = 0x100
+free_action2_ids = list(range(0, total_action2_ids))
+
+"""
+Statistics about spritegroups.
+The 1st field of type C{int} contains the largest number of concurrently active spritegroup ids.
+The 2nd field of type L{Position} contains a positional reference to the last spritegroup of the concurrently active ones.
+"""
+spritegroup_stats = (0, None)
+
+total_tmp_locations = 0x7F
+
+"""
+Statistics about temporary Action2 registers.
+The 1st field of type C{int} contains the largest number of concurrently active register ids.
+The 2nd field of type L{Position} contains a positional reference to the spritegroup.
+"""
+a2register_stats = (0, None)
+
+def print_stats():
+    """
+    Print statistics about used ids.
+    """
+    if spritegroup_stats[0] > 0:
+        generic.print_info("Concurrent spritegroups: {}/{} ({})".format(spritegroup_stats[0], total_action2_ids, str(spritegroup_stats[1])))
+    if a2register_stats[0] > 0:
+        generic.print_info("Concurrent Action2 registers: {}/{} ({})".format(a2register_stats[0], total_tmp_locations, str(a2register_stats[1])))
 
 class Action2(base_action.BaseAction):
     """
     Abstract Action2 base class.
 
-    @ivar name; Name of the action2.
+    @ivar name: Name of the action2.
     @type name: C{str}
 
     @ivar feature: Action2 feature byte.
     @type feature: C{int}
 
+    @ivar pos: Position reference to source.
+    @type pos: L{Position}
+
     @ivar num_refs: Number of references to this action2.
     @type num_refs: C{int}
 
@@ -43,30 +72,37 @@ class Action2(base_action.BaseAction):
                          to be used in this varaction2.
     @type tmp_locations: C{list} of C{int}
     """
-    def __init__(self, feature, name):
+    def __init__(self, feature, name, pos):
         self.feature = feature
         self.name = name
+        self.pos = pos
         self.num_refs = 0
         self.id = None
         self.references = []
         #0x00 - 0x7F: available to user
-        #0x80 - 0xFF: used by NML
+        #0x80 - 0xFE: used by NML
+        #0xFF: Used for some house variables
         #0x100 - 0x10F: Special meaning (used for some CB results)
-        self.tmp_locations = list(range(0x80, 0x100))
+        self.tmp_locations = list(range(0x80, 0x80 + total_tmp_locations))
 
-    def prepare_output(self):
+    def prepare_output(self, sprite_num):
         free_references(self)
 
-        if self.num_refs == 0:
-            self.id = free_action2_ids[0]
-        else:
-            try:
+        global spritegroup_stats
+
+        try:
+            if self.num_refs == 0:
+                self.id = free_action2_ids[0]
+            else:
                 self.id = free_action2_ids.pop()
-            except IndexError:
-                raise generic.ScriptError("Unable to allocate ID for [random]switch, sprite set/layout/group or produce-block. Try reducing the number of such blocks.")
+                num_used = total_action2_ids - len(free_action2_ids)
+                if num_used > spritegroup_stats[0]:
+                    spritegroup_stats = (num_used, self.pos)
+        except IndexError:
+            raise generic.ScriptError("Unable to allocate ID for [random]switch, sprite set/layout/group or produce-block. Try reducing the number of such blocks.", self.pos)
 
     def write_sprite_start(self, file, size):
-        assert self.num_refs == 0, "Action2 reference counting has %d dangling references." % self.num_refs
+        assert self.num_refs == 0, "Action2 reference counting has {:d} dangling references.".format(self.num_refs)
         file.comment("Name: " + self.name)
         file.start_sprite(size + 3)
         file.print_bytex(2)
@@ -97,8 +133,14 @@ class Action2(base_action.BaseAction):
                                 also for 'chained' action2s.
         @type force_recursive: C{bool}
         """
+        global a2register_stats
+
         if location in self.tmp_locations:
             self.tmp_locations.remove(location)
+            num_used = total_tmp_locations - len(self.tmp_locations)
+            if num_used > a2register_stats[0]:
+                a2register_stats = (num_used, self.pos)
+
         for act2_ref in self.references:
             if force_recursive or act2_ref.is_proc:
                 act2_ref.action2.remove_tmp_location(location, True)
@@ -135,7 +177,6 @@ def add_ref(ref, source_action, reference_as_proc = False):
     """
 
     # Add reference to list of references of the source action
-    if ref.name.value == 'CB_FAILED': return
     act2 = ref.act2 if ref.act2 is not None else resolve_spritegroup(ref.name).get_action2(source_action.feature)
     source_action.references.append(Action2Reference(act2, reference_as_proc))
     act2.num_refs += 1
@@ -230,7 +271,7 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
             This method should not be called, because calling a method on a meta class can be troublesome.
             Instead, call initialize(..).
             """
-            raise NotImplementedError('__init__ must be implemented in ASTSpriteGroup-subclass %r, initialize(..) should be called instead' % type(self))
+            raise NotImplementedError('__init__ must be implemented in ASTSpriteGroup-subclass {:r}, initialize(..) should be called instead'.format(type(self)))
 
         def initialize(self, name = None, feature = None, num_params = 0):
             """
@@ -273,7 +314,7 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
             if (not cls_is_relocatable) and cls_is_referenced:
                 register_spritegroup(self)
 
-        def prepare_output(self):
+        def prepare_act2_output(self):
             """
             Prepare this node for outputting.
             This sets the feature and makes sure it is correct.
@@ -287,7 +328,7 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
                 # copy, since we're going to modify
                 ref_nodes = self._referencing_nodes.copy()
                 for node in ref_nodes:
-                    used = node.prepare_output()
+                    used = node.prepare_act2_output()
                     if not used:
                         node._remove_reference(self)
 
@@ -297,8 +338,12 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
                     assert len(self.feature_set) == 1
                     for n in self._referencing_nodes:
                         if n.feature_set != self.feature_set:
-                            raise generic.ScriptError("Cannot refer to block '%s' with feature '%s', expected feature is '%s'" % \
-                                    (self.name.value, general.feature_name(self.feature_set.copy().pop()), general.feature_name(n.feature_set.difference(self.feature_set).pop())), n.pos)
+                            msg = "Cannot refer to block '{}' with feature '{}', expected feature is '{}'"
+                            msg = msg.format(self.name.value,
+                                    general.feature_name(next(iter(self.feature_set))),
+                                    general.feature_name(n.feature_set.difference(self.feature_set).pop()))
+                            raise generic.ScriptError(msg, n.pos)
+
                 elif len(self._referencing_nodes) != 0:
                     for n in self._referencing_nodes:
                         # Add the features from all calling blocks to the set
@@ -307,7 +352,7 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
                 if len(self._referencing_nodes) == 0:
                     # if we can be 'not used', there ought to be a way to refer to this block
                     assert self.name is not None
-                    generic.print_warning("Block '%s' is not referenced, ignoring." % self.name.value, self.pos)
+                    generic.print_warning("Block '{}' is not referenced, ignoring.".format(self.name.value), self.pos)
 
             return len(self._referencing_nodes) != 0
 
@@ -339,7 +384,7 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
             @return: A collection containing all links to other nodes.
             @rtype: C{iterable} of L{SpriteGroupRef}
             """
-            raise NotImplementedError('collect_references must be implemented in ASTSpriteGroup-subclass %r' % type(self))
+            raise NotImplementedError('collect_references must be implemented in ASTSpriteGroup-subclass {:r}'.format(type(self)))
 
         def set_action2(self, action2, feature):
             """
@@ -388,17 +433,22 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
             """
 
             if target_ref.name.value == "CB_FAILED": return
+
             target = resolve_spritegroup(target_ref.name)
             if target.is_spriteset():
                 assert target.num_params == 0
                 # Referencing a spriteset directly from graphics/[random]switch
                 # Passing parameters is not possible here
                 if len(target_ref.param_list) != 0:
-                    raise generic.ScriptError("Passing parameters to '%s' is only possible from a spritelayout." % target_ref.name.value, target_ref.pos)
+                    raise generic.ScriptError("Passing parameters to '{}' is only possible from a spritelayout.".format(target_ref.name.value), target_ref.pos)
+
                 self.used_sprite_sets.append(target)
             else:
                 if len(target_ref.param_list) != target.num_params:
-                    raise generic.ScriptError("'%s' expects %d parameters, encountered %d." % (target_ref.name.value, target.num_params, len(target_ref.param_list)), target_ref.pos)
+                    msg = "'{}' expects {:d} parameters, encountered {:d}."
+                    msg = msg.format(target_ref.name.value, target.num_params, len(target_ref.param_list))
+                    raise generic.ScriptError(msg, target_ref.pos)
+
                 self._referenced_nodes.add(target)
                 target._referencing_nodes.add(self)
 
@@ -435,7 +485,7 @@ def register_spritegroup(spritegroup):
     """
     name = spritegroup.name.value
     if name in spritegroup_list:
-        raise generic.ScriptError("Block with name '%s' has already been defined" % name, spritegroup.pos)
+        raise generic.ScriptError("Block with name '{}' has already been defined".format(name), spritegroup.pos)
     spritegroup_list[name] = spritegroup
     nml.global_constants.spritegroups[name] = name
 
@@ -450,6 +500,6 @@ def resolve_spritegroup(name):
     @rtype: L{ASTSpriteGroup}
     """
     if name.value not in spritegroup_list:
-        raise generic.ScriptError("Unknown identifier encountered: '%s'" % name.value, name.pos)
+        raise generic.ScriptError("Unknown identifier encountered: '{}'".format(name.value), name.pos)
     return spritegroup_list[name.value]
 
