@@ -18,8 +18,8 @@ from nml.actions import action2, action1
 from nml.ast import general
 
 class Action2Real(action2.Action2):
-    def __init__(self, feature, name, loaded_list, loading_list):
-        action2.Action2.__init__(self, feature, name)
+    def __init__(self, feature, name, pos, loaded_list, loading_list):
+        action2.Action2.__init__(self, feature, name, pos)
         self.loaded_list = loaded_list
         self.loading_list = loading_list
 
@@ -43,7 +43,7 @@ def get_real_action2s(spritegroup, feature):
     actions = []
 
     if feature not in action2.features_sprite_group:
-        raise generic.ScriptError("Sprite groups that combine sprite sets are not supported for feature '%s'." % general.feature_name(feature), spritegroup.pos)
+        raise generic.ScriptError("Sprite groups that combine sprite sets are not supported for feature '{}'.".format(general.feature_name(feature)), spritegroup.pos)
 
     # First make sure that all referenced real sprites are put in a single action1
     spriteset_list = []
@@ -56,7 +56,9 @@ def get_real_action2s(spritegroup, feature):
         if view_names != sorted(['loading', 'loaded']):
             raise generic.ScriptError("Expected a 'loading' and a 'loaded' (list of) sprite set(s).", spritegroup.pos)
     elif feature in (0x05, 0x0B, 0x0D, 0x10):
-        generic.print_warning("Sprite groups for feature %02X will not be supported in the future, as they are no longer needed. Directly refer to sprite sets instead." % feature, spritegroup.pos)
+        msg = "Sprite groups for feature {:02X} will not be supported in the future, as they are no longer needed. Directly refer to sprite sets instead."
+        msg = msg.format(feature)
+        generic.print_warning(msg, spritegroup.pos)
         if view_names != ['default']:
             raise generic.ScriptError("Expected only a 'default' (list of) sprite set(s).", spritegroup.pos)
 
@@ -69,10 +71,31 @@ def get_real_action2s(spritegroup, feature):
             if view.name.value == 'loading': loading_list.append(action1_index)
             else: loaded_list.append(action1_index)
 
-    actions.append(Action2Real(feature, spritegroup.name.value + (" - feature %02X" % feature), loaded_list, loading_list))
+    actions.append(Action2Real(feature, spritegroup.name.value + " - feature {:02X}".format(feature), spritegroup.pos, loaded_list, loading_list))
     spritegroup.set_action2(actions[-1], feature)
     return actions
 
+def make_simple_real_action2(feature, name, pos, action1_index):
+    """
+    Make a simple real action2, referring to only 1 action1 sprite set
+
+    @param feature: Feature of the needed action2
+    @type feature: C{int}
+
+    @param name: Name of the action2
+    @type name: C{str}
+
+    @param pos: Positional context.
+    @type  pos: L{Position}
+
+    @param action1_index: Index of the action1 sprite set to use
+    @type action1_index: C{int}
+
+    @return: The created real action2
+    @rtype: L{Action2Real}
+    """
+    return Action2Real(feature, name, pos, [action1_index], [action1_index] if feature <= 0x03 else [])
+
 def create_spriteset_actions(spritegroup):
     """
     Create action2s for directly-referenced sprite sets
@@ -87,12 +110,13 @@ def create_spriteset_actions(spritegroup):
     # Iterate over features first for more efficient action1s
     for feature in spritegroup.feature_set:
         if len(spritegroup.used_sprite_sets) != 0 and feature not in action2.features_sprite_group:
-            raise generic.ScriptError("Directly referring to sprite sets is not possible for feature %02X" % feature, spritegroup.pos)
+            raise generic.ScriptError("Directly referring to sprite sets is not possible for feature {:02X}".format(feature), spritegroup.pos)
         for spriteset in spritegroup.used_sprite_sets:
             if spriteset.has_action2(feature): continue
             action_list.extend(action1.add_to_action1([spriteset], feature, spritegroup.pos))
-            action1_index = action1.get_action1_index(spriteset)
-            real_action2 = Action2Real(feature, spriteset.name.value + (" - feature %02X" % feature), [action1_index], [action1_index] if feature <= 0x03 else [])
+
+            real_action2 = make_simple_real_action2(feature, spriteset.name.value + " - feature {:02X}".format(feature), spritegroup.pos, action1.get_action1_index(spriteset))
             action_list.append(real_action2)
             spriteset.set_action2(real_action2, feature)
     return action_list
+
