@@ -13,7 +13,7 @@ You should have received a copy of the GNU General Public License along
 with NML; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."""
 
-from nml import generic, expression, global_constants
+from nml import generic, expression, global_constants, nmlop
 from nml.actions import base_action, action0, action2, action2real, action2var, action3_callbacks, action6, actionD
 
 class Action3(base_action.BaseAction):
@@ -26,6 +26,9 @@ class Action3(base_action.BaseAction):
     @ivar id: Item ID of the item that this action3 represents.
     @type id: C{int}
 
+    @ivar is_livery_override: Whether this action 3 is a livery override
+    @type is_livery_override: C{bool}
+
     @ivar cid_mappings: List of mappings that map cargo IDs to Action2s.
     @type cid_mappings: C{list} of C{tuple}: (C{int}, L{Expression} before prepare_output, C{int} afterwards, C{str})
 
@@ -35,13 +38,14 @@ class Action3(base_action.BaseAction):
     @ivar references: All Action2s that are referenced by this Action3.
     @type references: C{list} of L{Action2Reference}
     """
-    def __init__(self, feature, id):
+    def __init__(self, feature, id, is_livery_override):
         self.feature = feature
         self.id = id
+        self.is_livery_override = is_livery_override
         self.cid_mappings = []
         self.references = []
 
-    def prepare_output(self):
+    def prepare_output(self, sprite_num):
         action2.free_references(self)
         map_cid = lambda cid: cid.get_action2_id(self.feature) if isinstance(cid, expression.SpriteGroupRef) else cid.value | 0x8000
         self.cid_mappings = [(cargo, map_cid(cid), comment) for cargo, cid, comment in self.cid_mappings]
@@ -74,7 +78,7 @@ class Action3(base_action.BaseAction):
 # Make sure all action2s created here get a unique name
 action2_id = 0
 
-def create_intermediate_varaction2(feature, varact2parser, mapping, default):
+def create_intermediate_varaction2(feature, varact2parser, mapping, default, pos):
     """
     Create a varaction2 based on a parsed expression and a value mapping
 
@@ -90,6 +94,9 @@ def create_intermediate_varaction2(feature, varact2parser, mapping, default):
     @param default: Default sprite group if no value matches
     @type default: L{SpriteGroupRef}
 
+    @param pos: Positional context.
+    @type  pos: L{Position}
+
     @return: A tuple containing the action list and a reference to the created action2
     @rtype: C{tuple} of (C{list} of L{BaseAction}, L{SpriteGroupRef})
     """
@@ -100,9 +107,9 @@ def create_intermediate_varaction2(feature, varact2parser, mapping, default):
     for mod in varact2parser.mods:
         act6.modify_bytes(mod.param, mod.size, mod.offset + 4)
 
-    name = expression.Identifier("@action3_%d" % action2_id)
+    name = expression.Identifier("@action3_{:d}".format(action2_id))
     action2_id += 1
-    varaction2 = action2var.Action2Var(feature, name.value, 0x89)
+    varaction2 = action2var.Action2Var(feature, name.value, pos, 0x89)
     varaction2.var_list = varact2parser.var_list
     offset = 5 + varact2parser.var_list_size
     for proc in varact2parser.proc_call_list:
@@ -120,7 +127,7 @@ def create_intermediate_varaction2(feature, varact2parser, mapping, default):
                 if return_value.name.value == 'CB_FAILED':
                     result, comment = action2var.parse_result(return_value, action_list, act6, offset, varaction2, None, 0x89)
                 else:
-                    extra_actions, result, comment = create_proc_call_varaction2(feature, return_value, ret_value_function)
+                    extra_actions, result, comment = create_proc_call_varaction2(feature, return_value, ret_value_function, pos)
                     action_list.extend(extra_actions)
             else:
                 return_value = ret_value_function(return_value).reduce()
@@ -137,7 +144,7 @@ def create_intermediate_varaction2(feature, varact2parser, mapping, default):
     action_list.append(varaction2)
     return (action_list, return_ref)
 
-def create_proc_call_varaction2(feature, proc, ret_value_function):
+def create_proc_call_varaction2(feature, proc, ret_value_function, pos):
     """
     Create a varaction2 that executes a procedure call and applies a function on the result
 
@@ -150,6 +157,9 @@ def create_proc_call_varaction2(feature, proc, ret_value_function):
     @param ret_value_function: Function to apply on the result (L{Expression} -> L{Expression})
     @type ret_value_function: C{function}
 
+    @param pos: Positional context.
+    @type  pos: L{Position}
+
     @return: A list of extra actions, reference to the created action2 and a comment to add
     @rtype: C{tuple} of (C{list} of L{BaseAction}, L{SpriteGroupRef}, C{str})
     """
@@ -158,22 +168,19 @@ def create_proc_call_varaction2(feature, proc, ret_value_function):
 
     mapping = {0xFFFF: (expression.SpriteGroupRef(expression.Identifier('CB_FAILED'), [], None), None)}
     default = ret_value_function(expression.Variable(expression.ConstantNumeric(0x1C)))
-    action_list, result = create_intermediate_varaction2(feature, varact2parser, mapping, default)
+    action_list, result = create_intermediate_varaction2(feature, varact2parser, mapping, default, pos)
     comment = result.name.value + ';'
     return (action_list, result, comment)
 
-def create_cb_choice_varaction2(feature, var_num, and_mask, mapping, default):
+def create_cb_choice_varaction2(feature, expr, mapping, default, pos):
     """
     Create a varaction2 that maps callback numbers to various sprite groups
 
     @param feature: Feature of the varaction2
     @type feature: C{int}
 
-    @param var_num: Number of the variable to evaluate
-    @type var_num: C{int}
-
-    @param and_mask: And-mask to use for the variable
-    @type and_mask: C{int}
+    @param expr: Expression to evaluate
+    @type expr: L{Expression}
 
     @param mapping: Mapping of various values to sprite groups, with a possible extra function to apply to the return value
     @type mapping: C{dict} that maps C{int} to C{tuple} of (L{SpriteGroupRef}, C{function}, or C{None})
@@ -181,31 +188,67 @@ def create_cb_choice_varaction2(feature, var_num, and_mask, mapping, default):
     @param default: Default sprite group if no value matches
     @type default: L{SpriteGroupRef}
 
+    @param pos: Positional context.
+    @type  pos: L{Position}
+
     @return: A tuple containing the action list and a reference to the created action2
     @rtype: C{tuple} of (C{list} of L{BaseAction}, L{SpriteGroupRef})
     """
     varact2parser = action2var.Varaction2Parser(feature)
-    varact2parser.parse_expr(expression.Variable(expression.ConstantNumeric(var_num), mask = expression.ConstantNumeric(and_mask)))
-    return create_intermediate_varaction2(feature, varact2parser, mapping, default)
+    varact2parser.parse_expr(expr)
+    return create_intermediate_varaction2(feature, varact2parser, mapping, default, pos)
 
 def create_action3(feature, id, action_list, act6, is_livery_override):
-    if isinstance(id, expression.ConstantNumeric):
-        act3 = Action3(feature, id.value)
+    # Vehicles use an extended byte
+    size = 2 if feature <= 3 else 1
+    offset = 4 if feature <= 3 else 3
+
+    id, offset = actionD.write_action_value(id, action_list, act6, offset, size)
+    return Action3(feature, id.value, is_livery_override)
+
+house_tiles = {
+    0 : 'n', # 1x1
+    2 : 'nw', # 2x1
+    3 : 'ne', # 1x2
+    4 : 'news', # 2x2
+}
+
+def parse_graphics_block(graphics_block, feature, id, size, is_livery_override = False):
+    """
+    Parse a graphics block (or livery override) into a list of actions, mainly action3
+
+    @param graphics_block: Graphics-block to parse
+    @type graphics_block: L{GraphicsBlock}
+
+    @param feature: Feature of the associated item
+    @type feature: C{int}
+
+    @param id: ID of the associated item
+    @type id: L{Expression}
+
+    @param size: Size of the associated item (relevant for houses only)
+    @type size: L{ConstantNumeric} or C{None}
+
+    @param is_livery_override: Whether this is a livery override instead of a normal graphics block
+    @type is_livery_override: C{bool}
+
+    @return: The resulting list of actions
+    @rtype: L{BaseAction}
+    """
+    action_list = action2real.create_spriteset_actions(graphics_block)
+    if feature == 0x07:
+        # Multi-tile houses need more work
+        size_bit = size.value if size is not None else 0
+        for i, tile in enumerate(house_tiles[size_bit]):
+            tile_id = id if i == 0 else expression.BinOp(nmlop.ADD, id, expression.ConstantNumeric(i, id.pos), id.pos).reduce()
+            action_list.extend(parse_graphics_block_single_id(graphics_block, feature, tile_id, is_livery_override, tile, id))
     else:
-        tmp_param, tmp_param_actions = actionD.get_tmp_parameter(id)
-        # Vehicles use an extended byte
-        size = 2 if feature <= 3 else 1
-        offset = 4 if feature <= 3 else 3
-        act6.modify_bytes(tmp_param, size, offset)
-        action_list.extend(tmp_param_actions)
-        act3 = Action3(feature, 0)
+        action_list.extend(parse_graphics_block_single_id(graphics_block, feature, id, is_livery_override))
+    return action_list
 
-    act3.is_livery_override = is_livery_override
-    return act3
-
-def parse_graphics_block(graphics_block, feature, id, is_livery_override = False):
+def parse_graphics_block_single_id(graphics_block, feature, id, is_livery_override, house_tile = None, house_north_tile_id = None):
     action6.free_parameters.save()
-    prepend_action_list = action2real.create_spriteset_actions(graphics_block)
+    prepend_action_list = []
     action_list = []
     act6 = action6.Action6()
     act3 = create_action3(feature, id, action_list, act6, is_livery_override)
@@ -222,7 +265,7 @@ def parse_graphics_block(graphics_block, feature, id, is_livery_override = False
             cb_table = action3_callbacks.callbacks[feature]
             if cb_name in cb_table:
                 if cb_name in seen_callbacks:
-                    raise generic.ScriptError("Callback '%s' is defined multiple times." % cb_name, cargo_id.pos)
+                    raise generic.ScriptError("Callback '{}' is defined multiple times.".format(cb_name), cargo_id.pos)
                 seen_callbacks.add(cb_name)
 
                 info_list = cb_table[cb_name]
@@ -230,16 +273,21 @@ def parse_graphics_block(graphics_block, feature, id, is_livery_override = False
                     info_list = [info_list]
 
                 for info in info_list:
+                    if 'deprecate_message' in info:
+                        generic.print_warning(info['deprecate_message'], cargo_id.pos)
+                    if house_tile is not None and 'tiles' in info and house_tile not in info['tiles']:
+                        continue
+
                     if info['type'] == 'cargo':
                         # Not a callback, but an alias for a certain cargo type
                         if info['num'] in cargo_gfx:
-                            raise generic.ScriptError("Graphics for '%s' are defined multiple times." % cb_name, cargo_id.pos)
-                        cargo_gfx[info['num']] = graphics.result
+                            raise generic.ScriptError("Graphics for '{}' are defined multiple times.".format(cb_name), cargo_id.pos)
+                        cargo_gfx[info['num']] = graphics.result.value
                     elif info['type'] == 'cb':
-                        callbacks.append( (info, graphics.result) )
+                        callbacks.append( (info, graphics.result.value) )
                     elif info['type'] == 'override':
                         assert livery_override is None
-                        livery_override = graphics.result
+                        livery_override = graphics.result.value
                     else:
                         assert False
                 continue
@@ -249,17 +297,18 @@ def parse_graphics_block(graphics_block, feature, id, is_livery_override = False
         # Raise the error only now, to let the 'unknown identifier' take precedence
         if feature >= 5: raise generic.ScriptError("Associating graphics with a specific cargo is possible only for vehicles and stations.", cargo_id.pos)
         if cargo_id.value in cargo_gfx:
-             raise generic.ScriptError("Graphics for cargo %d are defined multiple times." % cargo_id.value, cargo_id.pos)
-        cargo_gfx[cargo_id.value] = graphics.result
+            raise generic.ScriptError("Graphics for cargo {:d} are defined multiple times.".format(cargo_id.value), cargo_id.pos)
+        cargo_gfx[cargo_id.value] = graphics.result.value
 
     if graphics_block.default_graphics is not None:
         if 'default' not in action3_callbacks.callbacks[feature]:
-            raise generic.ScriptError("Default graphics may not be defined for this feature (0x%02X)." % feature, graphics_block.default_graphics.pos)
+            raise generic.ScriptError("Default graphics may not be defined for this feature (0x{:02X}).".format(feature), graphics_block.default_graphics.pos)
         if None in cargo_gfx:
             raise generic.ScriptError("Default graphics are defined twice.", graphics_block.default_graphics.pos)
-        cargo_gfx[None] = graphics_block.default_graphics
+        cargo_gfx[None] = graphics_block.default_graphics.value
 
-    if len(callbacks) != 0:
+    # An in-between varaction2 is always needed for houses
+    if len(callbacks) != 0 or feature == 0x07:
         cb_flags = 0
         # Determine the default value
         if None not in cargo_gfx:
@@ -280,7 +329,7 @@ def parse_graphics_block(graphics_block, feature, id, is_livery_override = False
                 cb_flags |= 1 << cb_info['flag_bit']
 
             value_function = cb_info.get('value_function', None)
-            mapping_val = (gfx, value_function) 
+            mapping_val = (gfx, value_function)
 
             # See action3_callbacks for info on possible values
             purchase = cb_info.get('purchase', 0)
@@ -308,39 +357,64 @@ def parse_graphics_block(graphics_block, feature, id, is_livery_override = False
 
         # Handle CB 36
         if len(cb36_mapping) != 0:
-            actions, cb36_ref = create_cb_choice_varaction2(feature, 0x10, 0xFF, cb36_mapping, default_val)
+            expr = expression.Variable(expression.ConstantNumeric(0x10), mask = expression.ConstantNumeric(0xFF))
+            actions, cb36_ref = create_cb_choice_varaction2(feature, expr, cb36_mapping, default_val, graphics_block.pos)
             prepend_action_list.extend(actions)
             cb_mapping[0x36] = (cb36_ref, None)
         if len(cb36_buy_mapping) != 0:
-            actions, cb36_ref = create_cb_choice_varaction2(feature, 0x10, 0xFF, cb36_buy_mapping, default_val)
+            expr = expression.Variable(expression.ConstantNumeric(0x10), mask = expression.ConstantNumeric(0xFF))
+            actions, cb36_ref = create_cb_choice_varaction2(feature, expr, cb36_buy_mapping, default_val, graphics_block.pos)
             prepend_action_list.extend(actions)
             cb_buy_mapping[0x36] = (cb36_ref, None)
         if len(prod_cb_mapping) != 0:
-            actions, cb_ref = create_cb_choice_varaction2(feature, 0x18, 0xFF, prod_cb_mapping, default_val)
+            expr = expression.Variable(expression.ConstantNumeric(0x18), mask = expression.ConstantNumeric(0xFF))
+            actions, cb_ref = create_cb_choice_varaction2(feature, expr, prod_cb_mapping, default_val, graphics_block.pos)
             prepend_action_list.extend(actions)
             cb_mapping[0x00] = (cb_ref, None)
 
-        for cargo in sorted(cargo_gfx):
+        for cargo in sorted(cargo_gfx, key=lambda x: -1 if x is None else x):
             mapping = cb_buy_mapping if cargo == 0xFF else cb_mapping
-            if len(mapping) == 0:
+            if len(mapping) == 0 and feature != 0x07:
                 # No callbacks here, so move along
+                # Except for houses, where we need to store some stuff in a register
                 continue
             if cargo_gfx[cargo] != default_val:
                 # There are cargo-specific graphics, be sure to handle those
                 # Unhandled callbacks should chain to the default, though
                 mapping = mapping.copy()
                 mapping[0x00] = (cargo_gfx[cargo], None)
-            actions, cb_ref = create_cb_choice_varaction2(feature, 0x0C, 0xFFFF, mapping, default_val)
+
+            expr = expression.Variable(expression.ConstantNumeric(0x0C), mask = expression.ConstantNumeric(0xFFFF))
+            if feature == 0x07:
+                # Store relative x/y, item id (of the north tile) and house tile (HOUSE_TILE_XX constant) in register FF
+                # Format: 0xIIHHYYXX: II: item ID, HH: house tile, YY: relative y, XX: relative x
+                lowbytes_dict = {
+                    'n' : 0x000000,
+                    'e' : 0x010100,
+                    'w' : 0x020001,
+                    's' : 0x030101,
+                }
+                lowbytes = expression.ConstantNumeric(lowbytes_dict[house_tile])
+                highbyte = expression.BinOp(nmlop.SHIFT_LEFT, house_north_tile_id, expression.ConstantNumeric(24))
+                register_FF = expression.BinOp(nmlop.OR, lowbytes, highbyte, lowbytes.pos).reduce()
+                register_FF = expression.BinOp(nmlop.STO_TMP, register_FF, expression.ConstantNumeric(0xFF))
+                expr = expression.BinOp(nmlop.VAL2, register_FF, expr, register_FF.pos)
+
+                if len(mapping) == 0:
+                    # mapping must not be empty
+                    mapping[0x00] = (default_val, None)
+
+            actions, cb_ref = create_cb_choice_varaction2(feature, expr, mapping, default_val, graphics_block.pos)
             prepend_action_list.extend(actions)
             cargo_gfx[cargo] = cb_ref
 
     # Make sure to sort to make the order well-defined
     offset = 7 if feature <= 3 else 5
-    for cargo_id in sorted(cargo_gfx):
-        if cargo_id is None: continue
+    for cargo_id in sorted(cg for cg in cargo_gfx if cg is not None):
         result, comment = action2var.parse_result(cargo_gfx[cargo_id], action_list, act6, offset + 1, act3, None, 0x89)
         act3.cid_mappings.append( (cargo_id, result, comment) )
         offset += 3
+
     if None in cargo_gfx:
         result, comment = action2var.parse_result(cargo_gfx[None], action_list, act6, offset, act3, None, 0x89)
         act3.def_cid = result
