@@ -13,7 +13,7 @@ You should have received a copy of the GNU General Public License along
 with NML; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."""
 
-from nml import expression, grfstrings
+from nml import expression, grfstrings, generic
 from nml.actions import base_action, action6, actionD
 
 class Action4(base_action.BaseAction):
@@ -34,7 +34,7 @@ class Action4(base_action.BaseAction):
     @type id: C{int}
 
     @ivar texts: List of strings to write
-    @type texts: C{list} of C{unicode}
+    @type texts: C{list} of C{str}
     """
     def __init__(self, feature, lang, size, id, texts):
         self.feature = feature
@@ -43,7 +43,7 @@ class Action4(base_action.BaseAction):
         self.id = id
         self.texts = texts
 
-    def prepare_output(self):
+    def prepare_output(self, sprite_num):
         #To indicate a word value, bit 7 of the lang ID must be set
         if self.size == 2: self.lang = self.lang | 0x80
 
@@ -73,8 +73,8 @@ string_ranges = {
     0xC4: {'random_id': False}, # Station class names
     0xC5: {'random_id': False}, # Station names
     0xC9: {'random_id': False}, # House name
-    0xD0: {'random_id': True, 'ids': list(range(0x3FF, -1, -1))}, # Misc. text ids, used for callbacks and such
-    0xDC: {'random_id': True, 'ids': list(range(0xFF, -1, -1))}, # Misc. persistent text ids, used to set properties
+    0xD0: {'random_id': True, 'total': 0x400, 'ids': list(range(0x3FF, -1, -1))}, # Misc. text ids, used for callbacks and such
+    0xDC: {'random_id': True, 'total': 0x100, 'ids': list(range(0xFF, -1, -1))}, # Misc. persistent text ids, used to set properties
 }
 
 # Mapping of string identifiers to D0xx/DCxx text ids
@@ -84,6 +84,16 @@ used_strings = {
     0xDC: {},
 }
 
+def print_stats():
+    """
+    Print statistics about used ids.
+    """
+    for t, l in string_ranges.items():
+        if l['random_id']:
+            num_used = l['total'] - len(l['ids'])
+            if num_used > 0:
+                generic.print_info("{:02X}xx strings: {}/{}".format(t, num_used, l['total']))
+
 def get_global_string_actions():
     """
     Get a list of global string actions
@@ -94,8 +104,8 @@ def get_global_string_actions():
     """
     texts = []
     actions = []
-    for string_range, strings in used_strings.iteritems():
-        for feature_name, id in strings.iteritems():
+    for string_range, strings in list(used_strings.items()):
+        for feature_name, id in list(strings.items()):
             feature, string_name = feature_name
             texts.append( (0x7F, (string_range << 8) | id, grfstrings.get_translation(string_name), feature) )
             for lang_id in grfstrings.get_translations(string_name):
@@ -164,7 +174,7 @@ def get_string_action4s(feature, string_range, string, id = None):
             id_val = id.value
         id_val = id_val | (string_range << 8)
     else:
-        # Not a string range, so we must have an idea
+        # Not a string range, so we must have an id
         assert id is not None
         size = 3 if feature <= 3 else 1
         if isinstance(id, expression.ConstantNumeric):
