@@ -13,6 +13,8 @@ You should have received a copy of the GNU General Public License along
 with NML; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."""
 
+from nml import generic
+
 class Assignment(object):
     """
     Simple storage container for a name / value pair.
@@ -34,14 +36,40 @@ class Assignment(object):
         self.pos = pos
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Assignment'
-        print (indentation+2)*' ' + 'Name:'
+        generic.print_dbg(indentation, 'Assignment')
+        generic.print_dbg(indentation + 2, 'Name:')
         self.name.debug_print(indentation + 4)
-        print (indentation+2)*' ' + 'Value:'
+
+        generic.print_dbg(indentation + 2, 'Value:')
         self.value.debug_print(indentation + 4)
 
     def __str__(self):
-        return "%s: %s;" % (str(self.name), str(self.value))
+        return "{}: {};".format(self.name, self.value)
+
+class UnitAssignment(Assignment):
+    """
+    Storage container where the value can have a unit.
+
+    @ivar unit: Unit of the value, or not C{None}
+    @type unit: L{Unit}
+    """
+    def __init__(self, name, value, unit, pos):
+        Assignment.__init__(self, name, value, pos)
+        self.unit = unit
+
+    def debug_print(self, indentation):
+        Assignment.debug_print(self, indentation)
+        generic.print_dbg(indentation + 2, 'Unit:')
+        if self.unit is None:
+            generic.print_dbg(indentation + 4, 'None')
+        else:
+            generic.print_dbg(indentation + 4, self.unit)
+
+    def __str__(self):
+        if self.unit is None:
+            return Assignment.__str__(self)
+        else:
+            return "{}: {} {};".format(self.name, self.value, self.unit.name)
 
 class Range(object):
     """
@@ -52,7 +80,7 @@ class Range(object):
     @ivar min: The minimum value of this range.
     @type min: L{Expression}
 
-    @ivar max: THe maximum value of this range.
+    @ivar max: The maximum value of this range.
     @type max: L{Expression} or C{None}
     """
     def __init__(self, min, max):
@@ -62,5 +90,5 @@ class Range(object):
     def __str__(self):
         if self.max is None:
             return str(self.min)
-        return "%s .. %s" % (str(self.min), str(self.max))
+        return "{} .. {}".format(self.min, self.max)
 
