@@ -44,13 +44,13 @@ class BaseCost(base_statement.BaseStatement):
                 elif cost.name.value in generic_base_costs:
                     #create temporary list, so it can be sorted for efficiency
                     tmp_list = []
-                    for num, type in base_cost_table.values():
+                    for num, type in list(base_cost_table.values()):
                         if type == cost.name.value:
                             tmp_list.append(assignment.Assignment(expression.ConstantNumeric(num), cost.value, cost.name.pos))
-                    tmp_list.sort(lambda x, y: cmp(x.name.value, y.name.value))
+                    tmp_list.sort(key=lambda x: x.name.value)
                     new_costs.extend(tmp_list)
                 else:
-                    raise generic.ScriptError("Unrecognized base cost identifier '%s' encountered" % cost.name.value, cost.name.pos)
+                    raise generic.ScriptError("Unrecognized base cost identifier '{}' encountered".format(cost.name.value), cost.name.pos)
             else:
                 cost.name = cost.name.reduce()
                 if isinstance(cost.name, expression.ConstantNumeric):
@@ -59,7 +59,7 @@ class BaseCost(base_statement.BaseStatement):
         self.costs = new_costs
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Base costs'
+        generic.print_dbg(indentation, 'Base costs')
         for cost in self.costs:
             cost.debug_print(indentation + 2)
 
@@ -69,7 +69,7 @@ class BaseCost(base_statement.BaseStatement):
     def __str__(self):
         ret = "basecost {\n"
         for cost in self.costs:
-            ret += "\t%s: %s;\n" % (str(cost.name), str(cost.value))
+            ret += "\t{}: {};\n".format(cost.name, cost.value)
         ret += "}\n"
         return ret
 
@@ -140,6 +140,11 @@ base_cost_table = {
     'PR_CLEAR_AQUEDUCT'             : (63, 'PR_CONSTRUCTION'),
     'PR_BUILD_LOCK'                 : (64, 'PR_CONSTRUCTION'),
     'PR_CLEAR_LOCK'                 : (65, 'PR_CONSTRUCTION'),
+    'PR_MAINTENANCE_RAIL'           : (66, 'PR_RUNNING'),
+    'PR_MAINTENANCE_ROAD'           : (67, 'PR_RUNNING'),
+    'PR_MAINTENANCE_CANAL'          : (68, 'PR_RUNNING'),
+    'PR_MAINTENANCE_STATION'        : (69, 'PR_RUNNING'),
+    'PR_MAINTENANCE_AIRPORT'        : (70, 'PR_RUNNING'),
 }
 
 generic_base_costs = ['PR_CONSTRUCTION', 'PR_RUNNING', 'PR_BUILD_VEHICLE']
