@@ -48,7 +48,7 @@ def feature_name(feature):
     @return: String with feature as used in nml files.
     @rtype: C{str}
     """
-    for name, num in feature_ids.iteritems():
+    for name, num in list(feature_ids.items()):
         if num == feature:
             return name
     assert False, "Invalid feature number"
@@ -64,8 +64,8 @@ def parse_feature(expr):
     @rtype: L{ConstantNumeric}
     """
     expr = expr.reduce_constant([feature_ids])
-    if expr.value not in feature_ids.values():
-        raise generic.ScriptError("Invalid feature '%02X' encountered." % expr.value, expr.pos)
+    if expr.value not in list(feature_ids.values()):
+        raise generic.ScriptError("Invalid feature '{:02X}' encountered.".format(expr.value), expr.pos)
     return expr
 
 class MainScript(base_statement.BaseStatementList):
@@ -75,7 +75,7 @@ class MainScript(base_statement.BaseStatementList):
                 base_statement.BaseStatementList.LIST_TYPE_NONE, statements, False, False, False, False)
 
     def __str__(self):
-        res = "" 
+        res = ""
         for stmt in self.statements:
             res += str(stmt) + '\n'
         return res
