@@ -19,6 +19,7 @@ from nml.actions import action0, action2, action2var, action3
 
 item_feature = None
 item_id = None
+item_size = None
 
 class Item(base_statement.BaseStatementList):
     """
@@ -30,79 +31,85 @@ class Item(base_statement.BaseStatementList):
     @type name: L{Identifier} or C{None} if N/A.
 
     @ivar id: Numeric ID of the item
-    @type id: C{int}
+    @type id: L{ConstantNumeric} or C{None} if N/A
+
+    @ivar size: Size, used by houses only
+    @type size: L{ConstantNumeric} or C{None} if N/A
     """
     def __init__(self, params, body, pos):
         base_statement.BaseStatementList.__init__(self, "item-block", pos, base_statement.BaseStatementList.LIST_TYPE_ITEM, body)
-        if len(params) >= 1:
-            self.feature = general.parse_feature(params[0])
-        else:
-            raise generic.ScriptError("Item block requires at least one parameter, got 0", self.pos)
-        if len(params) > 3:
-            raise generic.ScriptError("Item block requires at most 3 parameters, found %d" % len(params), self.pos)
 
-        self.id = params[2] if len(params) == 3 else None
+        if not (1 <= len(params) <= 4):
+            raise generic.ScriptError("Item block requires between 1 and 4 parameters, found {:d}.".format(len(params)), self.pos)
+        self.feature = general.parse_feature(params[0])
+        if self.feature.value in (0x08, 0x0C, 0x0E):
+            raise generic.ScriptError("Defining item blocks for this feature is not allowed.", self.pos)
         self.name = params[1] if len(params) >= 2 else None
 
+        self.id = params[2].reduce_constant(global_constants.const_list) if len(params) >= 3 else None
+        if isinstance(self.id, expression.ConstantNumeric) and self.id.value == -1:
+            self.id = None # id == -1 means default
+
+        if len(params) >= 4:
+            if self.feature.value != 0x07:
+                raise generic.ScriptError("item-block parameter 4 'size' may only be set for houses", params[3].pos)
+            self.size = params[3].reduce_constant(global_constants.const_list)
+            if self.size.value not in list(action0.house_sizes.keys()):
+                raise generic.ScriptError("item-block parameter 4 'size' does not have a valid value", self.size.pos)
+        else:
+            self.size = None
+
     def register_names(self):
-        if self.id:
-            self.id = self.id.reduce(global_constants.const_list)
         if self.name:
             if not isinstance(self.name, expression.Identifier):
                 raise generic.ScriptError("Item parameter 2 'name' should be an identifier", self.pos)
             if self.name.value in global_constants.item_names:
                 existing_id = global_constants.item_names[self.name.value].id
-                if not isinstance(existing_id, expression.ConstantNumeric):
-                    raise generic.ScriptError("Item with name '%s' has already been assigned a non-constant ID, extending this item definition is not possible." % self.name.value, self.pos)
-                if self.id is not None and (not isinstance(self.id, expression.ConstantNumeric) or existing_id.value != self.id.value):
-                    raise generic.ScriptError("Item with name '%s' has already been assigned to id %d, cannot reassign to id %d" % (self.name.value, existing_id.value, self.id.value), self.pos)
+                if self.id is not None and existing_id.value != self.id.value:
+                    raise generic.ScriptError("Duplicate item with name '{}'. This item has already been assigned to id {:d}, cannot reassign to id {:d}".format(self.name.value, existing_id.value, self.id.value), self.pos)
                 self.id = existing_id
 
+        # We may have to reserve multiple item IDs for houses
+        num_ids = action0.house_sizes[self.size.value] if self.size is not None else 1
         if self.id is None:
-            self.id = expression.ConstantNumeric(action0.get_free_id(self.feature.value))
+            self.id = expression.ConstantNumeric(action0.get_free_id(self.feature.value, num_ids, self.pos))
+        elif not action0.check_id_range(self.feature.value, self.id.value, num_ids, self.id.pos):
+            action0.mark_id_used(self.feature.value, self.id.value, num_ids)
         if self.name is not None:
             global_constants.item_names[self.name.value] = self
         base_statement.BaseStatementList.register_names(self)
 
     def pre_process(self):
-        global item_feature, item_id
+        global item_feature, item_id, item_size
         item_id = self.id
         item_feature = self.feature.value
+        item_size = self.size
         base_statement.BaseStatementList.pre_process(self)
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Item, feature', hex(self.feature.value)
+        generic.print_dbg(indentation, 'Item, feature', hex(self.feature.value))
         base_statement.BaseStatementList.debug_print(self, indentation + 2)
 
     def get_action_list(self):
-        global item_feature, item_id
+        global item_feature, item_id, item_size
         item_id = self.id
         item_feature = self.feature.value
+        item_size = self.size
         return base_statement.BaseStatementList.get_action_list(self)
 
     def __str__(self):
-        ret = 'item(%d' % self.feature.value
+        ret = 'item({:d}'.format(self.feature.value)
         if self.name is not None:
-            ret += ', %s' % str(self.name)
-        if self.id is not None:
-            ret += ', %s' % str(self.id)
+            ret += ', {}'.format(self.name)
+        ret += ', {}'.format(str(self.id) if self.id is not None else "-1")
+        if self.size is not None:
+            sizes =["1X1", None, "2X1", "1X2", "2X2"]
+            ret += ', HOUSE_SIZE_{}'.format(sizes[self.size.value])
         ret += ') {\n'
         ret += base_statement.BaseStatementList.__str__(self)
         ret += '}\n'
         return ret
 
-class Unit(object):
-    def __init__(self, name):
-        assert name in unit.units
-        self.name = name
-        self.type = unit.units[name]['type']
-        self.convert = unit.units[name]['convert']
-        self.ottd_mul = unit.units[name]['ottd_mul']
-        self.ottd_shift = unit.units[name]['ottd_shift']
-
-    def __str__(self):
-        return self.name
-
 class Property(object):
     """
     AST-node representing a single property. These are only valid
@@ -128,16 +135,14 @@ class Property(object):
 
     def pre_process(self):
         self.value = self.value.reduce(global_constants.const_list, unknown_id_fatal = False)
-        if (self.unit is not None and self.unit.type != 'nfo') and not (isinstance(self.value, expression.ConstantNumeric) or isinstance(self.value, expression.ConstantFloat)):
-            raise generic.ScriptError("Using a unit for a property is only allowed if the value is constant", self.pos)
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Property:', self.name.value
+        generic.print_dbg(indentation, 'Property:', self.name.value)
         self.value.debug_print(indentation + 2)
 
     def __str__(self):
         unit = '' if self.unit is None else ' ' + str(self.unit)
-        return '\t%s: %s%s;' % (self.name, self.value, unit)
+        return '\t{}: {}{};'.format(self.name, self.value, unit)
 
 class PropertyBlock(base_statement.BaseStatement):
     """
@@ -156,17 +161,17 @@ class PropertyBlock(base_statement.BaseStatement):
             prop.pre_process()
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Property block:'
+        generic.print_dbg(indentation, 'Property block:')
         for prop in self.prop_list:
             prop.debug_print(indentation + 2)
 
     def get_action_list(self):
-        return action0.parse_property_block(self.prop_list, item_feature, item_id)
+        return action0.parse_property_block(self.prop_list, item_feature, item_id, item_size)
 
     def __str__(self):
         ret = 'property {\n'
         for prop in self.prop_list:
-            ret += '%s\n' % str(prop)
+            ret += '{}\n'.format(prop)
         ret += '}\n'
         return ret
 
@@ -181,21 +186,22 @@ class LiveryOverride(base_statement.BaseStatement):
         pass
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Liverry override, wagon id:'
+        generic.print_dbg(indentation, 'Livery override, wagon id:')
         self.wagon_id.debug_print(indentation + 2)
         for graphics in self.graphics_block.graphics_list:
             graphics.debug_print(indentation + 2)
-        print (indentation+2)*' ' + 'Default graphics:', self.graphics_block.default_graphics
+        generic.print_dbg(indentation + 2, 'Default graphics:', self.graphics_block.default_graphics)
 
     def get_action_list(self):
         wagon_id = self.wagon_id.reduce_constant([(global_constants.item_names, global_constants.item_to_id)])
-        return action3.parse_graphics_block(self.graphics_block, item_feature, wagon_id, True)
+        return action3.parse_graphics_block(self.graphics_block, item_feature, wagon_id, item_size, True)
 
     def __str__(self):
-        ret = 'livery_override(%s) {\n' % str(self.wagon_id)
+        ret = 'livery_override({}) {{\n'.format(self.wagon_id)
         for graphics in self.graphics_block.graphics_list:
-            ret += "\t%s\n" % str(graphics)
-        if self.graphics_block.default_graphics is not None: ret += '\t%s;\n' % str(self.graphics_block.default_graphics)
+            ret += "\t{}\n".format(graphics)
+        if self.graphics_block.default_graphics is not None:
+            ret += '\t{}\n'.format(self.graphics_block.default_graphics)
         ret += '}\n'
         return ret
 
@@ -211,7 +217,9 @@ class GraphicsBlock(graphics_base_class):
         for graphics_def in self.graphics_list:
             graphics_def.reduce_expressions(item_feature)
         if self.default_graphics is not None:
-            self.default_graphics = action2var.reduce_varaction2_expr(self.default_graphics, item_feature)
+            if self.default_graphics.value is None:
+                raise generic.ScriptError("Returning the computed value is not possible in a graphics-block, as there is no computed value.", self.pos)
+            self.default_graphics.value = action2var.reduce_varaction2_expr(self.default_graphics.value, item_feature)
 
         # initialize base class and pre_process it as well (in that order)
         self.initialize(None, item_feature)
@@ -219,29 +227,30 @@ class GraphicsBlock(graphics_base_class):
 
     def collect_references(self):
         all_refs = []
-        for result in [g.result for g in self.graphics_list] + [self.default_graphics]:
-            if isinstance(result, expression.SpriteGroupRef): # Default may be None
-                all_refs.append(result)
+        for result in [g.result for g in self.graphics_list] + ([self.default_graphics] if self.default_graphics is not None else []):
+            if isinstance(result.value, expression.SpriteGroupRef):
+                all_refs.append(result.value)
         return all_refs
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Graphics block:'
+        generic.print_dbg(indentation, 'Graphics block:')
         for graphics in self.graphics_list:
             graphics.debug_print(indentation + 2)
         if self.default_graphics is not None:
-            print (indentation+2)*' ' + 'Default graphics:'
+            generic.print_dbg(indentation + 2, 'Default graphics:')
             self.default_graphics.debug_print(indentation + 4)
 
     def get_action_list(self):
-        if self.prepare_output():
-            return action3.parse_graphics_block(self, item_feature, item_id)
+        if self.prepare_act2_output():
+            return action3.parse_graphics_block(self, item_feature, item_id, item_size)
         return []
 
     def __str__(self):
         ret = 'graphics {\n'
         for graphics in self.graphics_list:
-            ret += "\t%s\n" % str(graphics)
-        if self.default_graphics is not None: ret += '\t%s;\n' % str(self.default_graphics)
+            ret += "\t{}\n".format(graphics)
+        if self.default_graphics is not None:
+            ret += '\t{}\n'.format(self.default_graphics)
         ret += '}\n'
         return ret
 
@@ -253,24 +262,16 @@ class GraphicsDefinition(object):
 
     def reduce_expressions(self, var_feature):
         # Do not reduce cargo-id (yet)
-        if self.result is None:
-            raise generic.ScriptError("Returning the computed value is not possible in a graphics-block, as there is no computed value.", self.cargo_id.pos)
-        self.result = action2var.reduce_varaction2_expr(self.result, var_feature)
+        if self.result.value is None:
+            raise generic.ScriptError("Returning the computed value is not possible in a graphics-block, as there is no computed value.", self.result.pos)
+        self.result.value = action2var.reduce_varaction2_expr(self.result.value, var_feature)
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Cargo ID:'
+        generic.print_dbg(indentation, 'Cargo ID:')
         self.cargo_id.debug_print(indentation + 2)
-        print indentation*' ' + 'Result:'
-        if self.result is not None:
-            self.result.debug_print(indentation + 2)
+        generic.print_dbg(indentation, 'Result:')
+        self.result.debug_print(indentation + 2)
 
     def __str__(self):
-        ret = str(self.cargo_id)
-        if self.result is None:
-            ret += ': return;'
-        elif isinstance(self.result, expression.SpriteGroupRef):
-            ret += ': %s;' % str(self.result)
-        else:
-            ret += ': return %s;' % str(self.result)
-        return ret
+        return '{}: {}'.format(self.cargo_id, self.result)
 
