@@ -23,6 +23,7 @@ class RailtypeTable(base_statement.BaseStatement):
         base_statement.BaseStatement.__init__(self, "rail type table", pos, False, False)
         self.railtype_list = railtype_list
         generic.OnlyOnce.enforce(self, "rail type table")
+        global_constants.is_default_railtype_table = False
         global_constants.railtype_table.clear()
 
     def register_names(self):
@@ -35,22 +36,22 @@ class RailtypeTable(base_statement.BaseStatement):
                         val_list.append(expression.StringLiteral(rt.value, rt.pos))
                     else:
                         val_list.append(rt)
-                    expression.parse_string_to_dword(val_list[-1])
+                    expression.parse_string_to_dword(val_list[-1]) # we don't care about the result, only validate the input
                 self.railtype_list[i] = val_list if len(val_list) > 1 else val_list[0]
             else:
                 name = railtype
                 if isinstance(railtype, expression.Identifier):
                     self.railtype_list[i] = expression.StringLiteral(railtype.value, railtype.pos)
-                expression.parse_string_to_dword(self.railtype_list[i])
+                expression.parse_string_to_dword(self.railtype_list[i]) # we don't care about the result, only validate the input
             global_constants.railtype_table[name.value] = i
 
     def pre_process(self):
         pass
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Railtype table'
+        generic.print_dbg(indentation, 'Railtype table')
         for railtype in self.railtype_list:
-            print (indentation+2)*' ' + 'Railtype: %s' % str(railtype.value)
+            generic.print_dbg(indentation + 2, 'Railtype: ', railtype.value)
 
     def get_action_list(self):
         return action0.get_railtypelist_action(self.railtype_list)
