@@ -27,7 +27,7 @@ class BinOp(Expression):
         self.expr2 = expr2
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Binary operator, op = ', self.op.token
+        generic.print_dbg(indentation, 'Binary operator, op =', self.op.token)
         self.expr1.debug_print(indentation + 2)
         self.expr2.debug_print(indentation + 2)
 
@@ -161,19 +161,20 @@ class BinOp(Expression):
 
     def supported_by_action2(self, raise_error):
         if not self.op.act2_supports:
-            token = " '%s'" % self.op.token if self.op.token else ""
-            if raise_error: raise generic.ScriptError("Operator%s not supported in a switch-block" % token, self.pos)
+            token = " '{}'".format(self.op.token) if self.op.token else ""
+            if raise_error: raise generic.ScriptError("Operator{} not supported in a switch-block".format(token), self.pos)
             return False
         return self.expr1.supported_by_action2(raise_error) and self.expr2.supported_by_action2(raise_error)
 
     def supported_by_actionD(self, raise_error):
         if not self.op.actd_supports:
-            token = " '%s'" % self.op.token if self.op.token else ""
             if raise_error:
                 if self.op == nmlop.STO_PERM: raise generic.ScriptError("STORE_PERM is only available in switch-blocks.", self.pos)
                 elif self.op == nmlop.STO_TMP: raise generic.ScriptError("STORE_TEMP is only available in switch-blocks.", self.pos)
+
                 #default case
-                raise generic.ScriptError("Operator%s not supported in parameter assignment" % token, self.pos)
+                token = " '{}'".format(self.op.token) if self.op.token else ""
+                raise generic.ScriptError("Operator{} not supported in parameter assignment".format(token), self.pos)
             return False
         return self.expr1.supported_by_actionD(raise_error) and self.expr2.supported_by_actionD(raise_error)
 
