@@ -19,31 +19,59 @@ from .string_literal import StringLiteral
 
 ignore_all_invalid_ids = False
 
+def default_id_func(x, pos):
+    """
+    Default id conversion function.
+
+    @param x: Value to convert.
+    @type  x: C{str}, C{int}, or C{float}
+
+    @param pos: Position of the id.
+    @type  pos: L{Position}
+
+    @return: Expression of the id.
+    @rtype:  L{Expression}
+    """
+    if isinstance(x, str):
+        return StringLiteral(x, pos)
+    else:
+        return ConstantNumeric(x, pos)
+
+
 class Identifier(Expression):
     def __init__(self, value, pos = None):
         Expression.__init__(self, pos)
         self.value = value
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'ID: ' + self.value
+        generic.print_dbg(indentation, 'ID:', self.value)
 
     def __str__(self):
         return self.value
 
     def reduce(self, id_dicts = [], unknown_id_fatal = True, search_func_ptr = False):
         for id_dict in id_dicts:
-            id_d, func = (id_dict, lambda x, pos: StringLiteral(x, pos) if isinstance(x, basestring) else ConstantNumeric(x, pos)) if not isinstance(id_dict, tuple) else id_dict
+            if isinstance(id_dict, tuple):
+                id_d, func = id_dict
+            else:
+                id_d, func = id_dict, default_id_func
+
             if self.value in id_d:
                 if search_func_ptr:
                     # Do not reduce function pointers, since they have no (numerical) value
                     return func(id_d[self.value], self.pos)
                 else:
                     return func(id_d[self.value], self.pos).reduce(id_dicts)
-        if unknown_id_fatal and not ignore_all_invalid_ids: raise generic.ScriptError("Unrecognized identifier '" + self.value + "' encountered", self.pos)
+
+        if unknown_id_fatal and not ignore_all_invalid_ids:
+            raise generic.ScriptError("Unrecognized identifier '" + self.value + "' encountered", self.pos)
+
         return self
 
     def supported_by_actionD(self, raise_error):
-        if raise_error: raise generic.ScriptError("Unknown identifier '%s'" % self.value, self.pos)
+        if raise_error:
+            raise generic.ScriptError("Unknown identifier '{}'".format(self.value), self.pos)
+
         return False
 
     def __eq__(self, other):
