@@ -30,6 +30,9 @@ constant_numbers = {
     #never expire
     'VEHICLE_NEVER_EXPIRES' : 0xFF,
 
+    #default_cargo
+    'DEFAULT_CARGO_FIRST_REFITTABLE' : 0xFF,
+
     #cargo classes
     'CC_PASSENGERS'   : 0,
     'CC_MAIL'         : 1,
@@ -47,8 +50,8 @@ constant_numbers = {
     'CC_NEO_BULK'     : 12,
     'CC_SPECIAL'      : 15,
     'NO_CARGO_CLASS'           : 0,
-    'ALL_NORMAL_CARGO_CLASSES' : 0x07FF,
-    'ALL_CARGO_CLASSES'        : 0x87FF,
+    'ALL_NORMAL_CARGO_CLASSES' : 0x1FFF,
+    'ALL_CARGO_CLASSES'        : 0x9FFF,
 
     #engine class
     'ENGINE_CLASS_STEAM'    : 0x00,
@@ -64,63 +67,6 @@ constant_numbers = {
     'RUNNING_COST_ROADVEH'  : 0x4C48,
     'RUNNING_COST_NONE'     : 0x0000,
 
-    #vehicle cb flags
-    'VEH_CBF_VISUAL_EFFECT_AND_POWERED' : 0, # trains
-    'VEH_CBF_VISUAL_EFFECT'             : 0, # rvs/ships
-    'VEH_CBF_WAGON_LENGTH'              : 1,
-    'VEH_CBF_LOAD_AMOUNT'               : 2,
-    'VEH_CBF_REFITTED_CAPACITY'         : 3,
-    'VEH_CBF_ARTICULATED_PARTS'         : 4,
-    'VEH_CBF_CARGO_SUFFIX'              : 5,
-    'VEH_CBF_COLOUR_MAPPING'            : 6,
-    'VEH_CBF_SOUND_EFFECT'              : 7,
-
-    #corresponding callbacks
-    'VEH_CB_VISUAL_EFFECT_AND_POWERED'  : 0x10, # trains
-    'VEH_CB_VISUAL_EFFECT'              : 0x10, # rvs/ships
-    'VEH_CB_WAGON_LENGTH'               : 0x11,
-    'VEH_CB_LOAD_AMOUNT'                : 0x12,
-    'VEH_CB_REFITTED_CAPACITY'          : 0x15,
-    'VEH_CB_ARTICULATED_PARTS'          : 0x16,
-    'VEH_CB_CARGO_SUFFIX'               : 0x19,
-    'VEH_CB_CAN_ATTACH_WAGON'           : 0x1D,
-    'VEH_CB_TEXT_PURCHASE_SCREEN'       : 0x23,
-    'VEH_CB_COLOUR_MAPPING'             : 0x2D,
-    'VEH_CB_START_STOP_CHECK'           : 0x31,
-    'VEH_CB_32DAY'                      : 0x32,
-    'VEH_CB_SOUND_EFFECT'               : 0x33,
-    'VEH_CB_AUTOREPLACE_SELECT'         : 0x34,
-    'VEH_CB_VEHICLE_PROPERTIES'         : 0x36,
-
-    #properties for callback 0x36
-    'PROP_TRAINS_SPEED'                         : 0x09,
-    'PROP_TRAINS_POWER'                         : 0x0B,
-    'PROP_TRAINS_RUNNING_COST_FACTOR'           : 0x0D,
-    'PROP_TRAINS_CARGO_CAPACITY'                : 0x14,
-    'PROP_TRAINS_WEIGHT'                        : 0x16,
-    'PROP_TRAINS_COST_FACTOR'                   : 0x17,
-    'PROP_TRAINS_TRACTIVE_EFFORT_COEFFICIENT'   : 0x1F,
-    'PROP_TRAINS_BITMASK_VEHICLE_INFO'          : 0x25,
-
-    'PROP_ROADVEHS_RUNNING_COST_FACTOR'         : 0x09,
-    'PROP_ROADVEHS_CARGO_CAPACITY'              : 0x0F,
-    'PROP_ROADVEHS_COST_FACTOR'                 : 0x11,
-    'PROP_ROADVEHS_POWER'                       : 0x13,
-    'PROP_ROADVEHS_WEIGHT'                      : 0x14,
-    'PROP_ROADVEHS_SPEED'                       : 0x15,
-    'PROP_ROADVEHS_TRACTIVE_EFFORT_COEFFICIENT' : 0x18,
-
-    'PROP_SHIPS_COST_FACTOR'                    : 0x0A,
-    'PROP_SHIPS_SPEED'                          : 0x0B,
-    'PROP_SHIPS_CARGO_CAPACITY'                 : 0x0D,
-    'PROP_SHIPS_RUNNING_COST_FACTOR'            : 0x0F,
-
-    'PROP_AIRCRAFT_COST_FACTOR'                 : 0x0B,
-    'PROP_AIRCRAFT_SPEED'                       : 0x0C,
-    'PROP_AIRCRAFT_RUNNING_COST_FACTOR'         : 0x0E,
-    'PROP_AIRCRAFT_PASSENGER_CAPACITY'          : 0x0F,
-    'PROP_AIRCRAFT_MAIL_CAPACITY'               : 0x11,
-
     #shorten factor
     'SHORTEN_TO_8_8' : 0x00,
     'SHORTEN_TO_7_8' : 0x01,
@@ -131,15 +77,36 @@ constant_numbers = {
     'SHORTEN_TO_2_8' : 0x06,
     'SHORTEN_TO_1_8' : 0x07,
 
-    #visual effect / wagon power
+    #vehicle length
+    'VEHICLE_LENGTH' : 8,
+
+    #visual effect
     'VISUAL_EFFECT_DEFAULT'  : 0x00,
     'VISUAL_EFFECT_STEAM'    : 0x10,
     'VISUAL_EFFECT_DIESEL'   : 0x20,
     'VISUAL_EFFECT_ELECTRIC' : 0x30,
     'VISUAL_EFFECT_DISABLE'  : 0x40,
+
+    #effect spawn model
+    'EFFECT_SPAWN_MODEL_NONE'    : 0x40,
+    'EFFECT_SPAWN_MODEL_STEAM'   : 0x41,
+    'EFFECT_SPAWN_MODEL_DIESEL'  : 0x42,
+    'EFFECT_SPAWN_MODEL_ELECTRIC': 0x43,
+
+    #visual effect / effect spawn model
     'ENABLE_WAGON_POWER'     : 0x00,
     'DISABLE_WAGON_POWER'    : 0x80,
 
+    #create_effect
+    'EFFECT_SPRITE_STEAM'                    : 0xF1,
+    'EFFECT_SPRITE_DIESEL'                   : 0xF2,
+    'EFFECT_SPRITE_ELECTRIC'                 : 0xF3,
+    'EFFECT_SPRITE_AIRCRAFT_BREAKDOWN_SMOKE' : 0xFA,
+    'EFFECT_SPRITE_NONE'                     : 0xFF,
+
+    'CB_RESULT_CREATE_EFFECT_CENTER'         : 0x2000,
+    'CB_RESULT_CREATE_EFFECT_NO_ROTATION'    : 0x4000,
+
     #train misc flags
     'TRAIN_FLAG_TILT' : 0,
     'TRAIN_FLAG_2CC'  : 1,
@@ -188,12 +155,12 @@ constant_numbers = {
     'AI_FLAG_CARGO'     : 0x00,
 
     # Callback results
-    'CB_RESULT_ATTACH_DISALLOW'            : 0xFD,
-    'CB_RESULT_ATTACH_ALLOW'               : 0xFE,
-    'CB_RESULT_ATTACH_ALLOW_IF_RAILTYPES'  : 0xFF,
+    'CB_RESULT_ATTACH_ALLOW_IF_RAILTYPES'  : 0x400,
+    'CB_RESULT_ATTACH_ALLOW'               : 0x401,
+    'CB_RESULT_ATTACH_DISALLOW'            : 0x402,
 
-    'CB_RESULT_NO_MORE_ARTICULATED_PARTS'  : 0xFF,
-    'CB_RESULT_REVERSED_VEHICLE'           : 0x80,
+    'CB_RESULT_NO_MORE_ARTICULATED_PARTS'  : 0x7FFF,
+    'CB_RESULT_REVERSED_VEHICLE'           : 0x4000,
 
     'CB_RESULT_32_DAYS_TRIGGER'            : 0,
     'CB_RESULT_32_DAYS_COLOUR_MAPPING'     : 1,
@@ -299,61 +266,29 @@ constant_numbers = {
 
     'NEW_CARGO_SPRITE'       : 0xFFFF,
 
-    #aircraft type/size
-    'AIRCRAFT_TYPE_NORMAL'     : 0x02,
+    #aircraft type
     'AIRCRAFT_TYPE_HELICOPTER' : 0x00,
-    'AIRCRAFT_SIZE_SMALL'      : 0x00,
-    'AIRCRAFT_SIZE_LARGE'      : 0x01,
+    'AIRCRAFT_TYPE_SMALL'      : 0x02,
+    'AIRCRAFT_TYPE_LARGE'      : 0x03,
 
     #ground sprite IDs
     'GROUNDSPRITE_CONCRETE'    : 1420,
     'GROUNDSPRITE_CLEARED'     : 3924,
     'GROUNDSPRITE_NORMAL'      : 3981,
     'GROUNDSPRITE_WATER'       : 4061,
+    'GROUNDSPRITE_SNOW_1_4'    : 4493,
+    'GROUNDSPRITE_SNOW_2_4'    : 4512,
+    'GROUNDSPRITE_SNOW_3_4'    : 4531,
+    'GROUNDSPRITE_SNOW_4_4'    : 4550,
     'GROUNDSPRITE_SNOW'        : 4550,
+    'GROUNDSPRITE_DESERT_1_2'  : 4512,
     'GROUNDSPRITE_DESERT'      : 4550,
 
     # general CB for rerandomizing
     'CB_RANDOM_TRIGGER'        : 0x01,
 
-    #house callback flags
-    'HOUSE_CBF_BUILD'               : 0,
-    'HOUSE_CBF_ANIM_NEXT_FRAME'     : 1,
-    'HOUSE_CBF_ANIM_CONROL'         : 2,
-    'HOUSE_CBF_CONSTRUCTION_ANIM'   : 3,
-    'HOUSE_CBF_COLOUR'              : 4,
-    'HOUSE_CBF_CARGO_AMOUNT_ACCEPT' : 5,
-    'HOUSE_CBF_ANIM_SPEED'          : 6,
-    'HOUSE_CBF_DESTRUCTION'         : 7,
-    'HOUSE_CBF_CARGO_TYPE_ACCEPT'   : 8,
-    'HOUSE_CBF_CARGO_PRODUCTION'    : 9,
-    'HOUSE_CBF_PROTECTION'          : 10,
-    'HOUSE_CBF_FOUNDATIONS'         : 11,
-    'HOUSE_CBF_AUTOSLOPE'           : 12,
-
-    #corresponding callbacks
-    'HOUSE_CB_BUILD'                : 0x17,
-    'HOUSE_CB_ANIM_NEXT_FRAME'      : 0x1A,
-    'HOUSE_CB_ANIM_CONTROL'         : 0x1B,
-    'HOUSE_CB_CONSTRUCTION_ANIM'    : 0x1C,
-    'HOUSE_CB_COLOUR'               : 0x1E,
-    'HOUSE_CB_CARGO_AMOUNT_ACCEPT'  : 0x1F,
-    'HOUSE_CB_ANIM_SPEED'           : 0x20,
-    'HOUSE_CB_DESTRUCTION'          : 0x21,
-    'HOUSE_CB_CARGO_TYPE_ACCEPT'    : 0x2A,
-    'HOUSE_CB_CARGO_PRODUCTION'     : 0x2E,
-    'HOUSE_CB_PROTECTION'           : 0x143,
-    'HOUSE_CB_ACCEPTED_CARGO'       : 0x148,
-    'HOUSE_CB_BUILDING_NAME'        : 0x14D,
-    'HOUSE_CB_FOUNDATIONS'          : 0x14E,
-    'HOUSE_CB_AUTOSLOPE'            : 0x14F,
-
     #house flags
-    'HOUSE_FLAG_SIZE_1x1'           : 0,
     'HOUSE_FLAG_NOT_SLOPED'         : 1,
-    'HOUSE_FLAG_SIZE_2x1'           : 2,
-    'HOUSE_FLAG_SIZE_1x2'           : 3,
-    'HOUSE_FLAG_SIZE_2x2'           : 4,
     'HOUSE_FLAG_ANIMATE'            : 5,
     'HOUSE_FLAG_CHURCH'             : 6,
     'HOUSE_FLAG_STADIUM'            : 7,
@@ -367,6 +302,21 @@ constant_numbers = {
     'HOUSE_ACCEPT_FOOD'             : 0x10, # 0x80 / 8
     'HOUSE_ACCEPT_FIZZY_DRINKS'     : 0x10, # 0x80 / 8
 
+    # house sizes
+    'HOUSE_SIZE_1X1'                : 0,
+    'HOUSE_SIZE_2X1'                : 2,
+    'HOUSE_SIZE_1X2'                : 3,
+    'HOUSE_SIZE_2X2'                : 4,
+
+    # house tiles
+    'HOUSE_TILE_NORTH'              : 0,
+    'HOUSE_TILE_EAST'               : 1,
+    'HOUSE_TILE_WEST'               : 2,
+    'HOUSE_TILE_SOUTH'              : 3,
+
+    # house callback results
+    'CB_RESULT_HOUSE_NO_MORE_PRODUCTION': 0x20FF,
+
     #town zones
     'TOWNZONE_EDGE'                 : 0,
     'TOWNZONE_OUTSKIRT'             : 1,
@@ -375,58 +325,9 @@ constant_numbers = {
     'TOWNZONE_CENTRE'               : 4,
     'ALL_TOWNZONES'                 : 0x1F,
 
-    #industry tile callback flags
-    'INDTILE_CBF_ANIM_NEXT_FRAME'     : 0,
-    'INDTILE_CBF_ANIM_SPEED'          : 1,
-    'INDTILE_CBF_CARGO_AMOUNT_ACCEPT' : 2,
-    'INDTILE_CBF_CARGO_TYPE_ACCEPT'   : 3,
-    'INDTILE_CBF_SLOPE_IS_SUITABLE'   : 4,
-    'INDTILE_CBF_FOUNDATIONS'         : 5,
-    'INDTILE_CBF_AUTOSLOPE'           : 6,
-
-    #corresponding callbacks
-    'INDTILE_CB_ANIM_CONTROL'         : 0x25,
-    'INDTILE_CB_ANIM_NEXT_FRAME'      : 0x26,
-    'INDTILE_CB_ANIM_SPEED'           : 0x27,
-    'INDTILE_CB_CARGO_AMOUNT_ACCEPT'  : 0x2B,
-    'INDTILE_CB_CARGO_TYPE_ACCEPT'    : 0x2C,
-    'INDTILE_CB_SLOPE_IS_SUITABLE'    : 0x2F,
-    'INDTILE_CB_FOUNDATIONS'          : 0x30,
-    'INDTILE_CB_AUTOSLOPE'            : 0x3C,
-
     #industry tile special flags
     'INDTILE_FLAG_RANDOM_ANIMATION'   : 0,
 
-    #industry callback flags
-    'IND_CBF_AVAILABILITY'          : 0,
-    'IND_CBF_PROD_CB_CARGO_ARRIVE'  : 1,
-    'IND_CBF_PROD_CB_256_TICKS'     : 2,
-    'IND_CBF_LOCATION_CHECK'        : 3,
-    'IND_CBF_RANDOM_PROD_CHANGE'    : 4,
-    'IND_CBF_MONTHLY_PROD_CHANGE'   : 5,
-    'IND_CBF_CARGO_SUBTYPE_DISPLAY' : 6,
-    'IND_CBF_EXTRA_TEXT_FUND'       : 7,
-    'IND_CBF_EXTRA_TEXT_INDUSTRY'   : 8,
-    'IND_CBF_CONTROL_SPECIAL'       : 9,
-    'IND_CBF_STOP_ACCEPT_CARGO'     : 10,
-    'IND_CBF_COLOUR'                : 11,
-    'IND_CBF_CARGO_INPUT'           : 12,
-    'IND_CBF_CARGO_OUTPUT'          : 13,
-
-    #corresponding callbacks
-    'IND_CB_AVAILABILITY'           : 0x22,
-    'IND_CB_LOCATION_CHECK'         : 0x28,
-    'IND_CB_RANDOM_PROD_CHANGE'     : 0x29,
-    'IND_CB_MONTHLY_PROD_CHANGE'    : 0x35,
-    'IND_CB_CARGO_SUBTYPE_DISPLAY'  : 0x37,
-    'IND_CB_EXTRA_TEXT_FUND'        : 0x38,
-    'IND_CB_EXTRA_TEXT_INDUSTRY'    : 0x3A,
-    'IND_CB_CONTROL_SPECIAL'        : 0x3B,
-    'IND_CB_STOP_ACCEPT_CARGO'      : 0x3D,
-    'IND_CB_COLOUR'                 : 0x14A,
-    'IND_CB_CARGO_INPUT'            : 0x14B,
-    'IND_CB_CARGO_OUTPUT'           : 0x14C,
-
     'CB_RESULT_IND_PROD_NO_CHANGE'      : 0x00,
     'CB_RESULT_IND_PROD_HALF'           : 0x01,
     'CB_RESULT_IND_PROD_DOUBLE'         : 0x02,
@@ -447,10 +348,10 @@ constant_numbers = {
     'CB_RESULT_IND_DO_NOT_USE_SPECIAL'  : 0x00,
     'CB_RESULT_IND_USE_SPECIAL'         : 0x01,
 
-    'CB_RESULT_IND_ALLOW'               : 0x00,
-    'CB_RESULT_IND_DISALLOW'            : 0x01,
+    'CB_RESULT_IND_NO_CONSTRUCTION'            : 0x0000,
+    'CB_RESULT_IND_PROBABILITY_FROM_PROPERTY'  : 0x0100,
 
-    'CB_RESULT_NO_TEXT'                 : 0xFF,
+    'CB_RESULT_NO_TEXT'                 : 0x400,
 
     'CB_RESULT_LOCATION_ALLOW'                        : 0x400,
     'CB_RESULT_LOCATION_DISALLOW'                     : 0x401,
@@ -503,50 +404,17 @@ constant_numbers = {
     'OBJ_FLAG_DRAW_WATER'      : 10,
     'OBJ_FLAG_ALLOW_BRIDGE'    : 11,
     'OBJ_FLAG_RANDOM_ANIMATION': 12,
+    'OBJ_FLAG_SCALE_BY_WATER'  : 13,
 
     #object animation triggers
     'OBJ_ANIM_IS_BUILT'        : 0,
     'OBJ_ANIM_PERIODIC'        : 1,
     'OBJ_ANIM_SYNC'            : 2,
 
-    #object callback flags
-    'OBJ_CBF_SLOPE_CHECK'      : 0,
-    'OBJ_CBF_ANIM_NEXT_FRAME'  : 1,
-    'OBJ_CBF_ANIM_SPEED'       : 2,
-    'OBJ_CBF_DECIDE_COLOUR'    : 3,
-    'OBJ_CBF_ADDITIONAL_TEXT'  : 4,
-    'OBJ_CBF_AUTOSLOPE'        : 5,
-
-    #corresponding callbacks
-    'OBJ_CB_SLOPE_CHECK'            : 0x157,
-    'OBJ_CB_ANIM_NEXT_FRAME'        : 0x158,
-    'OBJ_CB_ANIM_CONTROL'           : 0x159,
-    'OBJ_CB_ANIM_SPEED'             : 0x15A,
-    'OBJ_CB_DECIDE_COLOUR'          : 0x15B,
-    'OBJ_CB_ADDITIONAL_TEXT'        : 0x15C,
-    'OBJ_CB_AUTOSLOPE'              : 0x15D,
-
     # Special values for object var 0x60
     'OBJECT_TYPE_OTHER_GRF'     : 0xFFFE,
     'OBJECT_TYPE_NO_OBJECT'     : 0xFFFF,
 
-    #airport tile callback flags
-    'APT_CBF_ANIM_NEXT_FRAME'   : 0,
-    'APT_CBF_ANIM_SPEED'        : 1,
-    'APT_CBF_SLOPE_CHECK'       : 4,
-    'APT_CBF_FOUNDATIONS'       : 5,
-    'APT_CBF_AUTOSLOPE'         : 6,
-
-    #corresponding callbacks
-    'APT_CB_ANIM_CONTROL'       : 0x152,
-    'APT_CB_ANIM_NEXT_FRAME'    : 0x153,
-    'APT_CB_ANIM_SPEED'         : 0x154,
-    'APT_CB_FOUNDATIONS'        : 0x150,
-
-    #Airport callbacks
-    'AIRPORT_CB_ADDITIONAL_TEXT' : 0x155,
-    'AIRPORT_CB_LAYOUT_NAME'     : 0x156,
-
     #railtype flags
     'RAILTYPE_FLAG_CATENARY'          : 0,
     'RAILTYPE_FLAG_NO_LEVEL_CROSSING' : 1, # for OpenTTD > r20049
@@ -640,6 +508,11 @@ constant_numbers = {
     'PLAYERTYPE_HUMAN_IN_AI'             : 2,
     'PLAYERTYPE_AI_IN_HUMAN'             : 3,
 
+    # search criteria for house var 0x65
+    'SEARCH_HOUSE_BY_TYPE'               : 0,
+    'SEARCH_HOUSE_BY_CLASS'              : 1,
+    'SEARCH_HOUSE_BY_GRFID'              : 2,
+
     #build types (industry var 0xB3)
     'BUILDTYPE_UNKNOWN'                  : 0,
     'BUILDTYPE_GAMEPLAY'                 : 1,
@@ -652,7 +525,7 @@ constant_numbers = {
     'IND_CREATION_FUND'                  : 2,
     'IND_CREATION_PROSPECT'              : 3,
 
-    #airport types (vehicle var 0x44)
+    #airport types (vehicle var 0x44, base station var airport_type)
     'AIRPORTTYPE_SMALL'                  : 0,
     'AIRPORTTYPE_LARGE'                  : 1,
     'AIRPORTTYPE_HELIPORT'               : 2,
@@ -668,6 +541,15 @@ constant_numbers = {
     'DIRECTION_WEST'                      : 6,
     'DIRECTION_NORTHWEST'                 : 7,
 
+    'CANAL_DIRECTION_NORTH'               : 7,
+    'CANAL_DIRECTION_NORTHEAST'           : 0,
+    'CANAL_DIRECTION_EAST'                : 4,
+    'CANAL_DIRECTION_SOUTHEAST'           : 1,
+    'CANAL_DIRECTION_SOUTH'               : 5,
+    'CANAL_DIRECTION_SOUTHWEST'           : 2,
+    'CANAL_DIRECTION_WEST'                : 6,
+    'CANAL_DIRECTION_NORTHWEST'           : 3,
+
     'CORNER_W'                            : 0,
     'CORNER_S'                            : 1,
     'CORNER_E'                            : 2,
@@ -695,14 +577,16 @@ constant_numbers = {
     'SLOPE_STEEP_N'                       : 29,
 
     #loading stages
-    'LOADINGSTAGE_INITIALIZE'             : 0x0000,
-    'LOADINGSTAGE_RESERVE'                : 0x0101,
-    'LOADINGSTAGE_ACTIVATE'               : 0x0201,
-    'LOADINGSTAGE_TEST'                   : 0x0401,
+    'LOADING_STAGE_INITIALIZE'            : 0x0000,
+    'LOADING_STAGE_RESERVE'               : 0x0101,
+    'LOADING_STAGE_ACTIVATE'              : 0x0201,
+    'LOADING_STAGE_TEST'                  : 0x0401,
 
     #palettes
     'PALETTE_DOS'                           : 0,
+    'PALETTE_DEFAULT'                       : 0,
     'PALETTE_WIN'                           : 1,
+    'PALETTE_LEGACY'                        : 1,
 
     #game mode
     'GAMEMODE_MENU'                         : 0,
@@ -727,6 +611,25 @@ constant_numbers = {
     'MAP_TYPE_RECTANGULAR'                  : 1, #bit 0 set
     'MAP_TYPE_Y_BIGGER'                     : 2, #bit 0 clear, bit 1 set
 
+    # Platform types (platform_xx station variables)
+    'PLATFORM_SAME_STATION'                 : 0,
+    'PLATFORM_SAME_SECTION'                 : 1,
+    'PLATFORM_SAME_DIRECTION'               : 2,
+
+    # vehicle type (base station var 'had_vehicle_of_type')
+    'HAD_VEHICLE_OF_TYPE_TRAIN'             : 0,
+    'HAD_VEHICLE_OF_TYPE_BUS'               : 1,
+    'HAD_VEHICLE_OF_TYPE_TRUCK'             : 2,
+    'HAD_VEHICLE_OF_TYPE_AIRCRAFT'          : 3,
+    'HAD_VEHICLE_OF_TYPE_SHIP'              : 4,
+
+    # station facilities (base station var 'facilities')
+    'FACILITY_TRAIN'                        : 0,
+    'FACILITY_TRUCK_STOP'                   : 1,
+    'FACILITY_BUS_STOP'                     : 2,
+    'FACILITY_AIRPORT'                      : 3,
+    'FACILITY_DOCK'                         : 4,
+
     #Random triggers
     'TRIGGER_ALL_NEEDED'                    : 7,
 
@@ -758,6 +661,7 @@ constant_numbers = {
     'TILE_CLASS_TREES'                      : 0x04,
     'TILE_CLASS_STATION'                    : 0x05,
     'TILE_CLASS_WATER'                      : 0x06,
+    'TILE_CLASS_VOID'                       : 0x07,
     'TILE_CLASS_INDUSTRY'                   : 0x08,
     'TILE_CLASS_TUNNEL_BRIDGE'              : 0x09,
     'TILE_CLASS_OBJECTS'                    : 0x0A,
@@ -803,12 +707,6 @@ constant_numbers = {
     'CB_RESULT_AUTOSLOPE'                   : 0x00,  # callback 0x3C
     'CB_RESULT_NO_AUTOSLOPE'                : 0x01,  # callback 0x3C
 
-    #Zoom levels
-    'ZOOM_LEVEL_NORMAL'                     : 2,
-    'ZOOM_LEVEL_Z0'                         : 0,
-    'ZOOM_LEVEL_Z1'                         : 1,
-    'ZOOM_LEVEL_Z2'                         : 2,
-
     # Recolour modes for layout sprites
     'RECOLOUR_NONE'                         : 0,
     'RECOLOUR_TRANSPARENT'                  : 1,
@@ -1162,6 +1060,7 @@ misc_grf_bits = {
     'traffic_side'                       : {'param': 0x86, 'bit': 4},
     'desert_paved_roads'                 : {'param': 0x9E, 'bit': 1},
     'train_width_32_px'                  : {'param': 0x9E, 'bit': 3},
+    'second_rocky_tileset'               : {'param': 0x9E, 'bit': 6},
 }
 
 def add_1920(expr, info):
@@ -1180,7 +1079,7 @@ def add_1920(expr, info):
 
 def map_exponentiate(expr, info):
     """
-    Given a exponent, add an offset ot it and compute the exponentiation with base 2.
+    Given a exponent, add an offset to it and compute the exponentiation with base 2.
 
     @param expr: The exponent.
     @type expr: L{Expression}
@@ -1222,20 +1121,23 @@ def patch_variable(info, pos):
     return expression.SpecialParameter(generic.reverse_lookup(patch_variables, info), info, None, patch_variable_read, False, pos)
 
 patch_variables = {
-    'starting_year'   : {'num': 0x0B, 'start':  0, 'size': 32, 'function': add_1920},
-    'freight_trains'  : {'num': 0x0E, 'start':  0, 'size': 32},
-    'plane_speed'     : {'num': 0x10, 'start':  0, 'size': 32},
-    'base_sprite_2cc' : {'num': 0x11, 'start':  0, 'size': 32},
-    'map_type'        : {'num': 0x13, 'start': 24, 'size':  2},
-    'map_min_edge'    : {'num': 0x13, 'start': 20, 'size':  4, 'log_offset':  6, 'function': map_exponentiate},
-    'map_max_edge'    : {'num': 0x13, 'start': 16, 'size':  4, 'log_offset':  6, 'function': map_exponentiate},
-    'map_x_edge'      : {'num': 0x13, 'start': 12, 'size':  4, 'log_offset':  6, 'function': map_exponentiate},
-    'map_y_edge'      : {'num': 0x13, 'start':  8, 'size':  4, 'log_offset':  6, 'function': map_exponentiate},
-    'map_size'        : {'num': 0x13, 'start':  0, 'size':  8, 'log_offset': 12, 'function': map_exponentiate},
+    'starting_year'           : {'num': 0x0B, 'start':  0, 'size': 32, 'function': add_1920},
+    'freight_trains'          : {'num': 0x0E, 'start':  0, 'size': 32},
+    'plane_speed'             : {'num': 0x10, 'start':  0, 'size': 32},
+    'base_sprite_2cc'         : {'num': 0x11, 'start':  0, 'size': 32},
+    'map_type'                : {'num': 0x13, 'start': 24, 'size':  2},
+    'map_min_edge'            : {'num': 0x13, 'start': 20, 'size':  4, 'log_offset':  6, 'function': map_exponentiate},
+    'map_max_edge'            : {'num': 0x13, 'start': 16, 'size':  4, 'log_offset':  6, 'function': map_exponentiate},
+    'map_x_edge'              : {'num': 0x13, 'start': 12, 'size':  4, 'log_offset':  6, 'function': map_exponentiate},
+    'map_y_edge'              : {'num': 0x13, 'start':  8, 'size':  4, 'log_offset':  6, 'function': map_exponentiate},
+    'map_size'                : {'num': 0x13, 'start':  0, 'size':  8, 'log_offset': 12, 'function': map_exponentiate},
+    'max_height_level'        : {'num': 0x14, 'start':  0, 'size': 32},
+    'base_sprite_foundations' : {'num': 0x15, 'start':  0, 'size': 32},
+    'base_sprite_shores'      : {'num': 0x16, 'start':  0, 'size': 32},
 }
 
 def config_flag_read(bit, pos):
-    return expression.BinOp(nmlop.HASBIT, expression.Parameter(expression.ConstantNumeric(0x85), pos), expression.ConstantNumeric(bit), pos)
+    return expression.SpecialCheck((0x01, r'\70'), 0x85, (0, 1), bit, "PatchFlag({})".format(bit), varsize = 1, pos = pos)
 
 def config_flag(info, pos):
     return expression.SpecialParameter(generic.reverse_lookup(config_flags, info), info, None, config_flag_read, True, pos)
@@ -1251,6 +1153,7 @@ config_flags = {
     'signals_on_traffic_side' : 0x3B,
     'electrified_railways'    : 0x3C,
     'newhouses'               : 0x59,
+    'wagon_speed_limits'      : 0x5D,
     'newindustries'           : 0x67,
     'temperate_snowline'      : 0x6A,
     'newcargos'               : 0x6B,
@@ -1276,7 +1179,7 @@ def setting_from_info(info, pos):
 
 def item_to_id(item, pos):
     if not isinstance(item.id, expression.ConstantNumeric):
-        raise generic.ScriptError("Referencing item '%s' with a non-constant id is not possible." % item.name, pos)
+        raise generic.ScriptError("Referencing item '{}' with a non-constant id is not possible.".format(item.name), pos)
     return expression.ConstantNumeric(item.id.value, pos)
 
 def param_from_name(info, pos):
@@ -1286,6 +1189,7 @@ def create_spritegroup_ref(info, pos):
     return expression.SpriteGroupRef(expression.Identifier(info), [], pos)
 
 cargo_numbers = {}
+is_default_railtype_table = True
 railtype_table = {'RAIL': 0, 'ELRL': 1, 'MONO': 1, 'MGLV': 2}
 item_names = {}
 settings = {}
@@ -1306,3 +1210,13 @@ const_list = [
     (unified_maglev_var, unified_maglev),
     (spritegroups, create_spritegroup_ref),
 ]
+
+def print_stats():
+    """
+    Print statistics about used ids.
+    """
+    if len(cargo_numbers) > 0:
+        # Ids FE and FF have special meanings in Action3, so we do not consider them valid ids.
+        generic.print_info("Cargo translation table: {}/{}".format(len(cargo_numbers), 0xFE))
+    if not is_default_railtype_table:
+        generic.print_info("Railtype translation table: {}/{}".format(len(railtype_table), 0x100))
