@@ -13,43 +13,54 @@ You should have received a copy of the GNU General Public License along
 with NML; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."""
 
-from nml import generic, expression, tokens, nmlop
-from nml.ast import assignment, basecost, cargotable, conditional, deactivate, disable_item, error, font, general, grf, item, loop, produce, railtypetable, replace, spriteblock, switch, townnames, snowline, skipall, tilelayout, alt_sprites, base_sprites, override, sort_vehicles
+from nml import generic, expression, tokens, nmlop, unit
+from nml.ast import assignment, basecost, cargotable, conditional, deactivate, disable_item, error, font, general, grf, item, loop, produce, railtypetable, replace, spriteblock, switch, townnames, snowline, skipall, tilelayout, alt_sprites, base_graphics, override, sort_vehicles
 from nml.actions import actionD, real_sprite
 import ply.yacc as yacc
 
 class NMLParser(object):
+    """
+    @ivar lexer: Scanner providing tokens.
+    @type lexer: L{NMLLexer}
+
+    @ivar tokens: Tokens of the scanner (used by PLY).
+    @type tokens: C{List} of C{str}
+
+    @ivar parser: PLY parser.
+    @type parser: L{ply.yacc}
+    """
     def __init__(self):
         self.lexer = tokens.NMLLexer()
         self.lexer.build()
         self.tokens = self.lexer.tokens
-        self.parser = yacc.yacc(debug = False, module = self)
+        self.parser = yacc.yacc(debug = False, module = self, write_tables = 0)
 
-    def parse(self, text):
-        return self.parser.parse(text, lexer = self.lexer.lexer)
+    def parse(self, text, input_filename):
+        self.lexer.setup(text, input_filename)
+        return self.parser.parse(None, lexer = self.lexer.lexer)
 
 
     #operator precedence (lower in the list = higher priority)
     precedence = (
-        ('left','COMMA'),
-        ('right','TERNARY_OPEN','COLON'),
-        ('left','LOGICAL_OR'),
-        ('left','LOGICAL_AND'),
-        ('left','OR'),
-        ('left','XOR'),
-        ('left','AND'),
-        ('left','COMP_EQ','COMP_NEQ','COMP_LE','COMP_GE','COMP_LT','COMP_GT'),
-        ('left','SHIFT_LEFT','SHIFT_RIGHT','SHIFTU_RIGHT'),
-        ('left','PLUS','MINUS'),
-        ('left','TIMES','DIVIDE','MODULO'),
-        ('left','LOGICAL_NOT','BINARY_NOT'),
+        ('left', 'COMMA'),
+        ('right', 'TERNARY_OPEN','COLON'),
+        ('left', 'LOGICAL_OR'),
+        ('left', 'LOGICAL_AND'),
+        ('left', 'OR'),
+        ('left', 'XOR'),
+        ('left', 'AND'),
+        ('left', 'COMP_EQ', 'COMP_NEQ', 'COMP_LE', 'COMP_GE', 'COMP_LT', 'COMP_GT'),
+        ('left', 'SHIFT_LEFT', 'SHIFT_RIGHT', 'SHIFTU_RIGHT'),
+        ('left', 'PLUS', 'MINUS'),
+        ('left', 'TIMES', 'DIVIDE', 'MODULO'),
+        ('left', 'LOGICAL_NOT', 'BINARY_NOT'),
     )
 
     def p_error(self, t):
         if t is None:
             raise generic.ScriptError('Syntax error, unexpected end-of-file')
         else:
-            raise generic.ScriptError('Syntax error, unexpected token "%s"' % t.value, t.lineno)
+            raise generic.ScriptError('Syntax error, unexpected token "{}"'.format(t.value), t.lineno)
 
 
     #
@@ -91,7 +102,7 @@ class NMLParser(object):
                       | deactivate
                       | replace
                       | replace_new
-                      | base_sprites
+                      | base_graphics
                       | font_glyph
                       | alt_sprites
                       | snowline
@@ -241,11 +252,11 @@ class NMLParser(object):
 
     def p_setting_value(self, t):
         'setting_value : assignment'
-        t[0] =  t[1]
+        t[0] = t[1]
 
     def p_names_setting_value(self, t):
         'setting_value : ID COLON LBRACE name_string_list RBRACE SEMICOLON'
-        t[0] =  assignment.Assignment(t[1], t[4], t[1].pos)
+        t[0] = assignment.Assignment(t[1], t[4], t[1].pos)
 
     def p_name_string_list(self, t):
         '''name_string_list : name_string_item
@@ -294,6 +305,17 @@ class NMLParser(object):
                                    | generic_assignment_list generic_assignment'''
         t[0] = [] if len(t) == 1 else t[1] + [t[2]]
 
+    def p_snowline_assignment(self, t):
+        '''snowline_assignment : expression COLON expression SEMICOLON
+                               | expression COLON expression UNIT SEMICOLON'''
+        unit_value = None if len(t) == 5 else unit.get_unit(t[4])
+        t[0] = assignment.UnitAssignment(t[1], t[3], unit_value, t.lineno(1))
+
+    def p_snowline_assignment_list(self, t):
+        '''snowline_assignment_list :
+                                    | snowline_assignment_list snowline_assignment'''
+        t[0] = [] if len(t) == 1 else t[1] + [t[2]]
+
     #
     # Item blocks
     #
@@ -317,8 +339,8 @@ class NMLParser(object):
                                | NUMBER COLON expression SEMICOLON
                                | NUMBER COLON expression UNIT SEMICOLON'''
         name = t[1]
-        unit = None if len(t) == 5 else item.Unit(t[4])
-        t[0] = item.Property(name, t[3], unit, t.lineno(1))
+        unit_value = None if len(t) == 5 else unit.get_unit(t[4])
+        t[0] = item.Property(name, t[3], unit_value, t.lineno(1))
 
     def p_graphics_block(self, t):
         'graphics_block : GRAPHICS LBRACE graphics_list RBRACE'
@@ -330,15 +352,16 @@ class NMLParser(object):
 
     def p_graphics_list(self, t):
         '''graphics_list : graphics_assignment_list
-                         | graphics_assignment_list expression SEMICOLON
-                         | expression SEMICOLON'''
+                         | graphics_assignment_list switch_value
+                         | switch_value'''
         # Save graphics block as a tuple, we need to add position info later
         if len(t) == 2:
-            t[0] = (t[1], None)
-        elif len(t) == 4:
-            t[0] = (t[1], t[2])
+            if isinstance(t[1], list):
+                t[0] = (t[1], None)
+            else:
+                t[0] = ([], t[1])
         else:
-            t[0] = ([], t[1])
+            t[0] = (t[1], t[2])
 
     def p_graphics_assignment(self, t):
         'graphics_assignment : expression COLON switch_value'
@@ -383,8 +406,9 @@ class NMLParser(object):
         t[0] = switch.Switch(t[3], t[6], t.lineno(1))
 
     def p_switch_body(self, t):
-        'switch_body : switch_ranges switch_value'
-        t[0] = switch.SwitchBody(t[1], t[2])
+        '''switch_body : switch_ranges switch_value
+                       | switch_ranges'''
+        t[0] = switch.SwitchBody(t[1], t[2] if len(t) == 3 else None)
 
     def p_switch_ranges(self, t):
         '''switch_ranges :
@@ -402,9 +426,9 @@ class NMLParser(object):
         '''switch_value : RETURN expression SEMICOLON
                         | RETURN SEMICOLON
                         | expression SEMICOLON'''
-        if len(t) == 4: t[0] = t[2]
-        elif t[1] == 'return': t[0] = None
-        else: t[0] = t[1]
+        if len(t) == 4: t[0] = switch.SwitchValue(t[2], True, t[2].pos)
+        elif t[1] == 'return': t[0] = switch.SwitchValue(None, True, t.lineno(1))
+        else: t[0] = switch.SwitchValue(t[1], False, t[1].pos)
 
     def p_random_switch(self, t):
         'random_switch : RANDOMSWITCH LPAREN expression_list RPAREN LBRACE random_body RBRACE'
@@ -427,9 +451,9 @@ class NMLParser(object):
         '''real_sprite : LBRACKET expression_list RBRACKET
                        | ID COLON LBRACKET expression_list RBRACKET'''
         if len(t) == 4:
-            t[0] = real_sprite.RealSprite(t[2])
+            t[0] = real_sprite.RealSprite(param_list = t[2], poslist = [t.lineno(1)])
         else:
-            t[0] = real_sprite.RealSprite(t[4], t[1])
+            t[0] = real_sprite.RealSprite(param_list = t[4], label = t[1], poslist = [t.lineno(1)])
 
     def p_recolour_assignment_list(self, t):
         '''recolour_assignment_list :
@@ -449,8 +473,12 @@ class NMLParser(object):
         t[0] = assignment.Assignment(assignment.Range(t[1], t[3]), assignment.Range(t[5], None), t[1].pos)
 
     def p_recolour_sprite(self, t):
-        'real_sprite : RECOLOUR_SPRITE LBRACE recolour_assignment_list RBRACE'
-        t[0] = real_sprite.RecolourSprite(t[3])
+        '''real_sprite : RECOLOUR_SPRITE LBRACE recolour_assignment_list RBRACE
+                       | ID COLON RECOLOUR_SPRITE LBRACE recolour_assignment_list RBRACE'''
+        if len(t) == 5:
+            t[0] = real_sprite.RecolourSprite(mapping = t[3], poslist = [t.lineno(1)])
+        else:
+            t[0] = real_sprite.RecolourSprite(mapping = t[5], label = t[1], poslist = [t.lineno(1)])
 
     def p_template_declaration(self, t):
         'template_declaration : TEMPLATE ID LPAREN id_list RPAREN LBRACE spriteset_contents RBRACE'
@@ -484,11 +512,11 @@ class NMLParser(object):
         if len(t) == 9: t[0] = replace.ReplaceNewSprite(t[4], t[7], t[2], t.lineno(1))
         else: t[0] = replace.ReplaceNewSprite(t[3], t[6], None, t.lineno(1))
 
-    def p_base_sprites(self, t):
-        '''base_sprites : BASE_SPRITES LPAREN expression_list RPAREN LBRACE spriteset_contents RBRACE
-                        | BASE_SPRITES ID LPAREN expression_list RPAREN LBRACE spriteset_contents RBRACE'''
-        if len(t) == 9: t[0] = base_sprites.BaseSprite(t[4], t[7], t[2], t.lineno(1))
-        else: t[0] = base_sprites.BaseSprite(t[3], t[6], None, t.lineno(1))
+    def p_base_graphics(self, t):
+        '''base_graphics : BASE_GRAPHICS LPAREN expression_list RPAREN LBRACE spriteset_contents RBRACE
+                        | BASE_GRAPHICS ID LPAREN expression_list RPAREN LBRACE spriteset_contents RBRACE'''
+        if len(t) == 9: t[0] = base_graphics.BaseGraphics(t[4], t[7], t[2], t.lineno(1))
+        else: t[0] = base_graphics.BaseGraphics(t[3], t[6], None, t.lineno(1))
 
     def p_font_glyph(self, t):
         '''font_glyph : FONTGLYPH LPAREN expression_list RPAREN LBRACE spriteset_contents RBRACE
@@ -586,7 +614,7 @@ class NMLParser(object):
     # Snow line
     #
     def p_snowline(self, t):
-        'snowline : SNOWLINE LPAREN ID RPAREN LBRACE generic_assignment_list RBRACE'
+        'snowline : SNOWLINE LPAREN ID RPAREN LBRACE snowline_assignment_list RBRACE'
         t[0] = snowline.Snowline(t[3], t[6], t.lineno(1))
 
     #
