@@ -18,52 +18,62 @@ import ply.lex as lex
 from nml import expression, generic
 
 reserved = {
-    'grf' : 'GRF',
-    'var' : 'VARIABLE',
-    'param' : 'PARAMETER',
-    'cargotable' : 'CARGOTABLE',
-    'railtypetable' : 'RAILTYPETABLE',
-    'if' : 'IF',
-    'else' : 'ELSE',
-    'while' : 'WHILE', # reserved
-    'item' : 'ITEM', # action 0/3
-    'property' : 'PROPERTY',
-    'graphics' : 'GRAPHICS',
-    'snowline' : 'SNOWLINE',
-    'basecost' : 'BASECOST', 
-    'template' : 'TEMPLATE', #sprite template for action1
-    'spriteset' : 'SPRITESET', #action 1
-    'spritegroup' : 'SPRITEGROUP', #action 2
-    'switch' : 'SWITCH', #deterministic varaction2
-    'random_switch' : 'RANDOMSWITCH', #random action2
-    'produce' : 'PRODUCE', #production action2
-    'error' : 'ERROR', #action B
-    'disable_item' : 'DISABLE_ITEM',
-    'replace' : 'REPLACESPRITE', #action A
-    'replacenew' : 'REPLACENEWSPRITE', #action 5
-    'font_glyph' : 'FONTGLYPH', #action 12
-    'deactivate' : 'DEACTIVATE', #action E
-    'town_names' : 'TOWN_NAMES', # action F
-    'string' : 'STRING',
-    'return' : 'RETURN',
-    'livery_override' : 'LIVERYOVERRIDE',
-    'exit' : 'SKIP_ALL',
-    'tilelayout' : 'TILELAYOUT',
-    'spritelayout' : 'SPRITELAYOUT',
+    'grf' :                 'GRF',
+    'var' :                 'VARIABLE',
+    'param' :               'PARAMETER',
+    'cargotable' :          'CARGOTABLE',
+    'railtypetable' :       'RAILTYPETABLE',
+    'if' :                  'IF',
+    'else' :                'ELSE',
+    'while' :               'WHILE',            # reserved
+    'item' :                'ITEM',             # action 0/3
+    'property' :            'PROPERTY',
+    'graphics' :            'GRAPHICS',
+    'snowline' :            'SNOWLINE',
+    'basecost' :            'BASECOST',
+    'template' :            'TEMPLATE',         # sprite template for action1
+    'spriteset' :           'SPRITESET',        # action 1
+    'spritegroup' :         'SPRITEGROUP',      # action 2
+    'switch' :              'SWITCH',           # deterministic varaction2
+    'random_switch' :       'RANDOMSWITCH',     # random action2
+    'produce' :             'PRODUCE',          # production action2
+    'error' :               'ERROR',            # action B
+    'disable_item' :        'DISABLE_ITEM',
+    'replace' :             'REPLACESPRITE',    # action A
+    'replacenew' :          'REPLACENEWSPRITE', # action 5
+    'font_glyph' :          'FONTGLYPH',        # action 12
+    'deactivate' :          'DEACTIVATE',       # action E
+    'town_names' :          'TOWN_NAMES',       # action F
+    'string' :              'STRING',
+    'return' :              'RETURN',
+    'livery_override' :     'LIVERYOVERRIDE',
+    'exit' :                'SKIP_ALL',
+    'tilelayout' :          'TILELAYOUT',
+    'spritelayout' :        'SPRITELAYOUT',
     'alternative_sprites' : 'ALT_SPRITES',
-    'base_graphics' : 'BASE_SPRITES',
-    'recolour_sprite' : 'RECOLOUR_SPRITE',
-    'engine_override' : 'ENGINE_OVERRIDE',
-    'sort' : 'SORT_VEHICLES',
+    'base_graphics' :       'BASE_GRAPHICS',
+    'recolour_sprite' :     'RECOLOUR_SPRITE',
+    'engine_override' :     'ENGINE_OVERRIDE',
+    'sort' :                'SORT_VEHICLES',
 }
 
 line_directive1_pat = re.compile(r'\#line\s+(\d+)\s*(\r?\n|"(.*)"\r?\n)')
-line_directive2_pat = re.compile(r'\#\s+(\d+)\s+"(.*)"(\s+\d+\s*)?\r?\n')
+line_directive2_pat = re.compile(r'\#\s+(\d+)\s+"(.*)"\s*((?:\d+\s*)*)\r?\n')
 
 class NMLLexer(object):
+    """
+    @ivar lexer: PLY scanner object.
+    @type lexer: L{ply.lex}
+
+    @ivar includes: Stack of included files.
+    @type includes: C{List} of L{generic.LinePosition}
+
+    @ivar text: Input text to scan.
+    @type text: C{str}
+    """
 
     # Tokens
-    tokens = reserved.values() + [
+    tokens = list(reserved.values()) + [
         'ID',
         'PLUS',
         'MINUS',
@@ -153,7 +163,7 @@ class NMLLexer(object):
         return t
 
     def t_UNIT(self, t):
-        r'(nfo)|(mph)|(km/h)|(m/s)|(hpI)|(hpM)|(hp)|(kW)|(tons)|(ton)|(kg)'
+        r'(nfo)|(mph)|(km/h)|(m/s)|(hpI)|(hpM)|(hp)|(kW)|(tons)|(ton)|(kg)|(snow%)'
         return t
 
     def t_ID(self, t):
@@ -176,22 +186,49 @@ class NMLLexer(object):
         self.increment_lines(t.value.count("\n"))
 
     def t_ignore_whitespace(self, t):
-        "[ \t\r]"
+        "[ \t\r]+"
         pass
 
     def t_line_directive1(self, t):
         r'\#line\s+\d+\s*(\r?\n|".*"\r?\n)'
+        # See: https://gcc.gnu.org/onlinedocs/cpp/Line-Control.html
         m = line_directive1_pat.match(t.value)
         assert m is not None
         fname = self.lexer.lineno.filename if m.group(3) is None else m.group(3)
+
+        # This type of line directive contains no information about includes, so we have to make some assumptions
+        if self.includes and self.includes[-1].filename == fname:
+            # Filename equal to the one on top of the stack -> end of an include
+            self.includes.pop()
+        elif fname != self.lexer.lineno.filename:
+            # Not an end of include and not the current file -> start of an include
+            self.includes.append(self.lexer.lineno)
+
         self.set_position(fname, int(m.group(1), 10))
         self.increment_lines(t.value.count('\n') - 1)
 
     def t_line_directive2(self, t):
-        r'\#\s+\d+\s+".*"(\s+\d+\s*)?\r?\n'
+        r'\#\s+\d+\s+".*"\s*(\d+\s*)*\r?\n'
+        # Format: # lineno filename flags
+        # See: https://gcc.gnu.org/onlinedocs/cpp/Preprocessor-Output.html
         m = line_directive2_pat.match(t.value)
         assert m is not None
-        self.set_position(m.group(2), int(m.group(1), 10))
+        line, fname, flags = m.groups()
+        line = int(line, 10)
+        flags = [int(f, 10) for f in flags.split(" ") if f != ""] if flags is not None else []
+
+        if 1 in flags:
+            # File is being included, add current file/line to stack
+            self.includes.append(self.lexer.lineno)
+        elif 2 in flags:
+            # End of include, new file should be equal to the one on top of the stack
+            if self.includes and self.includes[-1].filename == fname:
+                self.includes.pop()
+            else:
+                # But of course user input can never be trusted
+                generic.print_warning("Information about included files is inconsistent, position information for errors may be wrong.")
+
+        self.set_position(fname, line)
         self.increment_lines(t.value.count('\n') - 1)
 
     def t_newline(self, t):
@@ -199,22 +236,45 @@ class NMLLexer(object):
         self.increment_lines(len(t.value))
 
     def t_error(self, t):
-        print "Illegal character '%s' at %s" % (t.value[0], t.lexer.lineno)
+        print(("Illegal character '{}' (character code 0x{:02X}) at {}, column {:d}".format(t.value[0], ord(t.value[0]), t.lexer.lineno, self.find_column(t))))
         sys.exit(1)
 
 
 
-    def build(self, **kwargs):
-        self.lexer = lex.lex(module=self, **kwargs)
-        self.set_position('input', 1)
+    def build(self):
+        """
+        Initial construction of the scanner.
+        """
+        self.lexer = lex.lex(module=self)
 
 
+    def setup(self, text, fname):
+        """
+        Setup scanner for scanning an input file.
+
+        @param text: Input text to scan.
+        @type  text: C{str}
+
+        @param fname: Filename associated with the input text (main input file).
+        @type  fname: C{str}
+        """
+        self.includes = []
+        self.text = text
+        self.set_position(fname, 1)
+        self.lexer.input(text)
+
     def set_position(self, fname, line):
         """
         @note: The lexer.lineno contains a Position object.
         """
-        self.lexer.lineno = generic.LinePosition(fname, line)
+        self.lexer.lineno = generic.LinePosition(fname, line, self.includes[:])
 
     def increment_lines(self, count):
         self.set_position(self.lexer.lineno.filename, self.lexer.lineno.line_start + count)
 
+    def find_column(self, t):
+        last_cr = self.text.rfind('\n', 0, t.lexpos)
+        if last_cr < 0: last_cr = 0
+        return t.lexpos - last_cr
+
+
